/*
 *  Copyright (c) 2004, The University Scheduler Project
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of the University Scheduler Project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

package edu.rpi.scheduler.engine;

import edu.rpi.scheduler.schedb.CourseDescriptor;
import edu.rpi.scheduler.schedb.DefaultSchedule;
import edu.rpi.scheduler.schedb.SectionDescriptor;
import edu.rpi.scheduler.schedb.UniqueSection;
import edu.rpi.scheduler.schedb.WeekMask;
import edu.rpi.scheduler.schedb.TermMask;
import edu.rpi.scheduler.schedb.YearMask;
import edu.rpi.scheduler.schedb.spec.*;

import java.util.*;

/**
 * Represents a set of courses, constraints, and, ultimately, schedules
 * generated by the user. A {@code Scheduler} holds the courses the user
 * wants to take, the sections he or she does not want, the time mask he or she
 * has blocked out, and more. It holds the list of {@code Department}s and
 * {@code Course}s and generates permutations of the selected courses based
 * on the user's constraints to produce {@code Schedule}s. This class also
 * sorts the schedules.
 */
public class SchedulerEngine {
    /**
     * A comparator that sorts {@code UniqueSection} instances in
     * increasing order by their {@code getLowestNumber()} return values.
     */
    private static final Comparator<UniqueSection> LOWNUM_COMPARATOR
            = new Comparator<UniqueSection>() {
        public int compare(UniqueSection o1, UniqueSection o2) {
            return o1.getLowestNumber().compareTo(o2.getLowestNumber());
        }
    };
    /**
     * A comparator that sorts <code>Map.Entry<?, Set></code> instances in
     * increasing order by how many elements are in the set.
     */
    private static final Comparator<Map.Entry<?,? extends Set<?>>> VALSIZE_COMPARATOR
            = new Comparator<Map.Entry<?,? extends Set<?>>>() {
        public int compare(Map.Entry<?,? extends Set<?>> o1, Map.Entry<?,? extends Set<?>> o2) {
            int s1 = o1.getValue().size();
            int s2 = o2.getValue().size();

            return s1 < s2 ? -1 : (s1 == s2 ? 0 : 1);
        }
    };



    private SchedulerDataPlugin schedulerPlugin = null;
    /**This is a complete set of schedule data**/
    private SchedulerData schedulerData = null;

    private Map<CourseDescriptor, SelectedCourse> selectedCourses
            = new LinkedHashMap<CourseDescriptor, SelectedCourse>();

    /**The time that is blocked out that we cannot use**/
    private YearMask<?> blockedWTime = null;
        /**The time that is blocked out that we cannot use**/
    
    /**A collection of blocked sections**/
    private Collection<SectionDescriptor> blockedSections
            = new ArrayList<SectionDescriptor>();
    /**
     * All the possible schedules that are unblocked.
     */
    private List<Schedule> possibleSchedules = Collections.emptyList();

    private boolean needsGenerating = true;
    /**
     * This keeps track of if we are currently in the process of recalculating our possible schedules or not. 
     */
    private volatile boolean working;

    private Comparator<? super Schedule> lastComparator = null;

    /**
     * Empty Constructor for the SchedulerEngine class
     */
    public SchedulerEngine() {
    }

    /**
     * Construct a scheduleEngine with the given schedulerData
     * @param schedulerData initial data (or data location) to use for this schedulerEngine
     */
    public SchedulerEngine(SchedulerData schedulerData) {
        setSchedulerData(schedulerData);
    }

    /**
     * Returns the SchedulerData being used by this ScheduleEngine
     * @return SchedulerData being used by this ScheduleEngine
     */
    public synchronized SchedulerData getSchedulerData() {
        return schedulerData;
    }

    /**
     * This function updates the schedulerData being used by this scheduler engine.
     * It also sets the schedulerPlugin and blockedTime based on the new schedulerData 
     * @param schedulerData the new SchedulerData to use for this schedulerEngine
     */
    public synchronized void setSchedulerData(SchedulerData schedulerData) {
        this.schedulerData = schedulerData;
        schedulerPlugin = schedulerData.getDataContext().getSchedulerPlugin();
        blockedWTime = schedulerPlugin.getTimeRepresentation().newYearMask();

    }

	/**
	 * This blocks all sections of a course that are in a given part of term.
	 * @param courseToBlock The course that we want to block
	 * @param PoTIdentifier The PartOfTerm during which we dont want this course
	 */
	public synchronized void BlockCourseByTerm(CourseDescriptor courseToBlock, String PoTIdentifier) {
        Collection<Section> sections = courseToBlock.getActualCourse().getSections();
		Collection<SectionDescriptor> blockedSections = new LinkedList<SectionDescriptor>(this.blockedSections); 
		for(Section s:sections){
			if(s.getPartOfTerm().equals(PoTIdentifier)){
				blockedSections.add(new SectionDescriptor(courseToBlock, s));
			}else if(s.getPartOfTerm().equals("Full Semester")){
				if(s.getTerm().equals(Year.getInstance().getTermThatContainsPoT(PoTIdentifier).getIdentifier())){
					blockedSections.add(new SectionDescriptor(courseToBlock, s));
				}
			}
		}
		setBlockedSections(blockedSections);
		setNeedsGenerating();
	}

	/**
	 * This unblocks all sections of a course that are in a given part of term.
	 *
	 * This actually just remves the course's sections for this term from the block list
	 * 
	 * @param courseToUnblock The course that we want to unblock
	 * @param PoTIdentifier The PartOfTerm during which we dont want this course
	 */
	public synchronized void UnblockCourseByTerm(CourseDescriptor courseToUnblock, String PoTIdentifier) {
		//First let's build a list of sectionDescriptors that we do not want to have in our blocked list
		Collection<Section> sections = courseToUnblock.getActualCourse().getSections();
		Collection<SectionDescriptor> unblockSections = new LinkedList<SectionDescriptor>();
		for(Section s:sections){
			if(s.getPartOfTerm().equals(PoTIdentifier)){
				unblockSections.add(new SectionDescriptor(courseToUnblock, s));
			}else if(s.getPartOfTerm().equals("Full Semester")){
				if(s.getTerm().equals(Year.getInstance().getTermThatContainsPoT(PoTIdentifier).getIdentifier())){
					unblockSections.add(new SectionDescriptor(courseToUnblock, s));
				}
			}
		}

		//then let's rebuild our blockedSection list making sure to not include the above generated sections
		Collection<SectionDescriptor> old = this.blockedSections;
		Collection<SectionDescriptor> blockedSections = new LinkedList<SectionDescriptor>();
		for(SectionDescriptor sd:old){
			if(!unblockSections.contains(sd)){
				blockedSections.add(sd);
			}
		}
		setBlockedSections(blockedSections);

		setNeedsGenerating();
	}

	
	/**
     * Sets the list of sections within the list of {@linkplain
     * #setSelectedCourses selected courses} that the user does not want to
     * be in. No schedules with these sections will be generated.
     *
     * If the new blockedSections is not the same as the old one, the {@code setNeedsGenerating()} 
     *
     * @param blockedSections the list of sections the user has blocked
     */
    public synchronized void setBlockedSections(Collection<SectionDescriptor> blockedSections) {
        Collection<SectionDescriptor> old = this.blockedSections;

        this.blockedSections = blockedSections;

        if (old.size() != blockedSections.size()
                || (!old.containsAll(blockedSections)
                && !blockedSections.containsAll(old))) {
            setNeedsGenerating();
        }
    }

    /**
     * Sets the time the user has blocked out, disallowing any classes during
     * this time. No schedules will be generated which overlap this time mask.
     * @param mask the time mask the user has blocked out
     */
    public synchronized void setBlockedTime(YearMask<?> mask) {
        YearMask<?> old = this.blockedWTime;

        YearMask<?> newBlocked = schedulerPlugin.getTimeRepresentation().newYearMask();
        newBlocked.merge(mask);
        this.blockedWTime = newBlocked;

        if (!newBlocked.equals(old)) {
            setNeedsGenerating();
        }
    }

    /**
     * Returns the time mask that the user has blocked out, disallowing any
     * classes during this time.
     * @return the time mask the user has blocked out
     */
    public synchronized YearMask<?> getBlockedTime() {
        YearMask<?> mask = schedulerPlugin.getTimeRepresentation().newYearMask();
        mask.merge(blockedWTime);
        return mask;
    }

    /**
     * Returns the list of sections the user has blocked.
     * @return the list of sections blocked by the user
     */
    public synchronized Collection<SectionDescriptor> getBlockedSections() {
        return blockedSections;
    }

    /**
     * Generates schedules from the given constraints, if necessary. If no
     * constraints have changed since the last call to
     * {@code generateSchedules} or if this function is already inuse ("working")
     * then this call will not do anything.
     *
     * Essentialy this function is a wrapper to call other functions that do the real work,
     * this function just keeps things in sync and going in the right order.
     */
    public synchronized void generateSchedules() {
        //excape if we are already working or if the schedule does not need generating.
        if (working || !needsGenerating) return;
        try {
            //flag that we are working
            working = true;

            //Really generate our schedules
            List<Schedule> schedules = reallyGenerateSchedules();

            //move things to an array list and sort things by the last comparitor
            possibleSchedules = new ArrayList<Schedule>(schedules);
            Comparator<? super Schedule> lastcomp = lastComparator;
            if (lastcomp != null) sortBy(lastcomp);
            //unflag that we need to generate the schedules
            needsGenerating = false;
        } finally {
            //if we successfuly finish, say we're done working.
            working = false;
        }
		//let's try to force Garbage collection
		System.gc();
		//make sure other things get updated
        fireGeneratedEvent();
    }

    private Set<EngineListener> listeners = new LinkedHashSet<EngineListener>();

    /**
     * Associate a listener with this SchedulerEngine
     * @param l EngineListener to add
     */
    public synchronized void addEngineListener(EngineListener l) {
        listeners.add(l);
    }

    /**
     * Remove a listener with this SchedulerEngine
     * @param l EngineListener to remove
     */
    public synchronized void removeEngineListener(EngineListener l) {
        listeners.remove(l);
    }

    /**
     * Shout out to our listener and say hey check me out!
     */
    private synchronized void fireGeneratedEvent() {
        for (EngineListener listener : listeners) {
            listener.schedulesGenerated(this);
        }
    }

    /**
     * Here we take our list of selected courses and create a list of potential schedules with them.
     * We start by generating a list of schedules with the required courses and then try to add the
     * extra courses.
     * @return A list of Schedule that contain all of the required courses and as many of the extra courses
     * as we could get to fit.
     */
    private synchronized List<Schedule> reallyGenerateSchedules() {
        /**
         * These are our possible schedules that work.
         * Things are added to this by the buildRequiredSchedules function
         */
        List<Schedule> schedules = new ArrayList<Schedule>();

        // First lets get a list of all our required courses
        List<CourseDescriptor> requiredCourses = getRequiredCourses();

        //If we have required courses we have to generate schedules with those first
        //If we cant build a schedule from the required courses we'll return an empty schedule
        if (!requiredCourses.isEmpty()) {
            List<Map.Entry<CourseDescriptor,SortedSet<UniqueSection>>> reqsections
                    = getGoodSections(requiredCourses);
            //now that we have the sections that will work, lets's build schedules
            buildRequiredSchedules(schedules, null, reqsections, 0);

            //if there were no schedules generated, let's return and empty schedule
            if (schedules.isEmpty()) return schedules;
        }

        //if we're here we either didnt have any required courses or we had some
        //valid schedules with all the required courses

        //now lets get our extra courses
        List<CourseDescriptor> extraCourses = getExtraCourses();

        //lets get the sections of extra courses that will work
        List<Map.Entry<CourseDescriptor,SortedSet<UniqueSection>>> optsections
                = getGoodSections(extraCourses);

        List<Schedule> optschedules = new ArrayList<Schedule>();

        //let's only try to add more schedules if our optional sections was not empty
        if (!optsections.isEmpty()) {
            //if there were no schedules already in existance, we wont try ADDING these to a schedule
            //we'll just make them.
            if (schedules.isEmpty()) {
                buildExtraSchedules(optschedules, null, optsections, 0);
            } else {
                //if there are alrady potential schedules loop over each one and try to
                //add in extras
                for (Schedule schedule : schedules) {
                    buildExtraSchedules(optschedules, schedule, optsections, 0);
                }
            }
        }

        //Here we combine our required only and our optional included schedules
        List<Schedule> totalSchedules = new ArrayList<Schedule>(schedules.size()
                + optschedules.size());
        totalSchedules.addAll(schedules);
        totalSchedules.addAll(optschedules);

        return totalSchedules;
    }

    /**
     * Return a Collection of the selected courses
     * @return ArrayList of SelectedCourses
     */
    public synchronized Collection<SelectedCourse> getSelectedCourses() {
        return new ArrayList<SelectedCourse>(selectedCourses.values());
    }

    /**
     * Pull all the CourseDescriptors in selectedCourses and return them in their own collection
     * @return all the CourseDescriptors in selectedCourses as a collection
     */
    public synchronized Collection<CourseDescriptor> getSelectedCourseDescriptors() {
        return new ArrayList<CourseDescriptor>(selectedCourses.keySet());
    }

    /**
     * Return a list of required courses.
     * @return List<CourseDescriptor> of required courses.
     */
    public List<CourseDescriptor> getRequiredCourses() {
        return getCoursesByExtraValue(false);
    }

    /**
     * Return a list of extra courses
     * @return List<CourseDescriptor> of extra courses.
     */
    public List<CourseDescriptor> getExtraCourses() {
        return getCoursesByExtraValue(true);
    }

    /**
     * Return courses that either do or do not have the extra flag set.
     * @param extra True will return a list of the courses listed as extra,
     * false will return a list of the rest of the courses
     * @return List of courses that are or are not extra
     */
    private synchronized List<CourseDescriptor> getCoursesByExtraValue(boolean extra) {
        List<CourseDescriptor> list
                = new ArrayList<CourseDescriptor>(selectedCourses.size());
        for (Map.Entry<CourseDescriptor, SelectedCourse> entry
                : selectedCourses.entrySet()) {
            if (entry.getValue().isExtra() == extra) list.add(entry.getKey());
        }
        return list;
    }

    /**
     * Take in a list of CourseDescriptors and return a list of maps of sorted sets maped to their
     * CourseDiscriptor.
     *
     * @param selected a list of mappings of CourseDecsriptors and a SortedSet of UniqueSections from which 
     * we want to weed out the blocked sections.
     *
     * @return A list of all the sections that are selected and not in the blockedSections 
     */
    private synchronized
    List<Map.Entry<CourseDescriptor,SortedSet<UniqueSection>>> getGoodSections(
            List<CourseDescriptor> selected) {

        Map<CourseDescriptor,SortedSet<UniqueSection>> sections
                = new HashMap<CourseDescriptor,SortedSet<UniqueSection>>(selected.size());
        /**
         * These are the sections that are blocked
         */
        Set<SectionDescriptor> blocked
                = new HashSet<SectionDescriptor>(blockedSections);

        // cache this so we don't keep acessing the field
        YearMask<?> blockedTime = this.blockedWTime;

        /**Loop over each course that has been selected**/
        for (CourseDescriptor course : selected) {
            //for each course that we are looping through get the sections
            Collection<Section> sects = course.getActualCourse().getSections();

            SortedSet<UniqueSection> good = new TreeSet<UniqueSection>(LOWNUM_COMPARATOR);

            Map<YearMask<?>,UniqueSection> unique= new HashMap<YearMask<?>, UniqueSection>(sects.size());
            sections.put(course, good);

            //Loop over each 'section' we just generated and add only the ones that work (not blocked)
            Outer: for (Section section : sects) {
                //Make sure we dont have any repeated sections by checking if current section
                //actually equals anything in the blocked list.
                for (SectionDescriptor sd : blocked) {
                    if (sd.getActualSection().equals(section)) continue Outer;
                }

                /**The mask that this section covers*/
                YearMask<?> mask = section.getYearMask();
                /**Get the unique section using the map that matches the secion map we just made*/
                UniqueSection sect = unique.get(mask);

                //See if this is the first section we are adding
                boolean first = sect == null;
                if (first) {
                    //if it is the first section we are adding make a new Unique Section Mask
                    sect = new UniqueSection(mask);
                    //and put a new mask on the section
                    unique.put(mask, sect);
                }

                //Build upon this UniqueSection by adding a section discriptor
                sect.addSection(new SectionDescriptor(course, section));

                //If this is the first section for this time mask and it fits into our empty spaces
                if (first && sect.getTimeMask().fitsInto(blockedTime)) {
                    good.add(sect);
                }
            }
        }

        //Finds all the unique sections within our sections variable
        Set<Map.Entry<CourseDescriptor,SortedSet<UniqueSection>>> entrySet = sections.entrySet();

        //makes a list of the unique sections so we can sort them by how many elesments are in the set
        List<Map.Entry<CourseDescriptor, SortedSet<UniqueSection>>> entries
                = new ArrayList<Map.Entry<CourseDescriptor,
                SortedSet<UniqueSection>>>(entrySet);

        Collections.sort(entries, VALSIZE_COMPARATOR);
        return entries;
    }

    /**
     * Build schedules consisting of required courses.
     *
     * This function works in recursive manner, adding to "schedules" (first param) each "schedule" (second param) once
     * the "schedule" (second param) is 'mature' (has every required course in it).
     *
     * Example (may not be necessary if you understand recursion but it's here nontheless):
     * Starting with required course 1 it loops over each of the sections seeing if it can be added.
     * If say section 2 works, it will call this function again with index 2 and an updated "schedule" (second param)
     * and try each section of course 2, with each working section triggering another call to this function with an
     * increased index so course 3 is checked.  Meanwile, the section checking loop inside this function continues
     * to check the next section and potentially call this function again if another section also works.
     * Once each of these branches has reached the end of the list of courses the now populated "schedule"
     * (second param) is added to the list of valid schedules.  If no working section is found for a
     * partciular course (because of the previous sequence of chosen sections) the section checking loop
     * simply runs out and the function is not called again and the "schedule" (second param) for that particular
     * branch is never added to the list of valid schedules (first param) (because, since we couldnt fit all
     * courses, it iself is not a valid schedules)
     *
     *
     *
     * @param schedules The list of potential schedules that we are currently working with.  (Stuff Gets added to this schedule)
     * @param schedule A partly created schedule to use as a base for this iteration's additions.
     * @param required List of CourseDescriptors and UniqueSections that we need to fit into this schedule.  This
     * list gets one smaller each recursive trip through this function
     * @param index This keeps track of what step of the recursion we are on so we know which course we are adding
     * on this loop through
     */
    private void buildRequiredSchedules(List<Schedule> schedules, DefaultSchedule schedule,
            List<Map.Entry<CourseDescriptor,SortedSet<UniqueSection>>> required,
            int index) {
        //Grab the scheudler data plugin for some reason
        SchedulerDataPlugin schedulerPlugin = this.schedulerPlugin;

        //Make sure each time we loop through this we are looking at the correct spot in the list of
        //required courses.  Each time we are looking at a diferent set of unique sections associated
        //with a particluar course
        Set<UniqueSection> reqsections = required.get(index).getValue();
        //increment our index so next time we call this function we've moved on to the next course's unique sections
        int newindex = index + 1;
        //make sure we dont run past the end.
        boolean last = newindex == required.size();

        //Loop through each unique section for the required course we are currently adding
        for (UniqueSection section : reqsections) {
            //if the schedule is not empty and this section cant be added, skip over this section
            //this is where we skip over sections that dont fit (have conflits)
            if (schedule != null && !schedule.canAdd(section)) {
                continue;
            }

            //copy the partly created schedule
            //I think we need to do this so we dont overwrite a schedule that may be used in another iteration
            DefaultSchedule sched;
            if (schedule == null) {
                sched = new DefaultSchedule(schedulerPlugin);
            } else {
                sched = new DefaultSchedule(schedulerPlugin, schedule);
            }

            //Since we already determined above that this section can be added, let's go ahead and add it to the copy
            //of our schedule
            sched.addSection(section);

            //If we are on the last iteration add it to our list
            if (last) schedules.add(sched);
            else buildRequiredSchedules(schedules, sched, required, newindex);
            /*otherwise let's keep building all possible schedules with this particular section for this particular
            course added.  Basicly this loops around and tries every other section for the next course, and for each
            one of those that works forks out another that keeps going.

            Since we only add the temporary schedule to the list when we've reached all the way to the base of the
            tree we know when we've gotten there it's a valid schedule that has all the required courses in it.
            */
        }
    }

    /**
     * Build schedules consisting of non courses.
     *
     * This function works in a nearly identical fashion to {@code buildRequiredSchedules()}.  See its documentation
     * for more info.  The big difference here is that every step of every branch of recursion that encounters a
     * valid section adds a schedule with this section added to the list.
     * of valid schedules
     *
     * @param schedules List of schedules that work.
     * @param schedule The partly built schedule to build upon during this iteration.
     * @param extra List of Courses and Unique sections to add to this schedule 
     * @param index What step of recursion we are on (so we know what course to try to add this time.
     */
    private void buildExtraSchedules(List<Schedule>  schedules, Schedule schedule,
            List<Map.Entry<CourseDescriptor,SortedSet<UniqueSection>>> extra,
            int index) {
        SchedulerDataPlugin schedulerPlugin = this.schedulerPlugin;

        //Get the sections for the course we are currently trying to add (on this step of recursion)
        Set<UniqueSection> extraSections = extra.get(index).getValue();
        //increment our index so we add the next course next time.
        int newindex = index + 1;
        //make sure we dont run off the end of our list of courses
        boolean last = newindex == extra.size();

        //Loop over each unique section for this course
        for (UniqueSection section : extraSections) {
            //if the schedule is not empty and this section cant be added, skip over this section
            //this is where we skip over sections that dont fit (have conflits)
            if (schedule != null && !schedule.canAdd(section)) {
                continue;
            }

            //Make a copy of our schedule so we dont mess up the copy that other branches of recursion
            //may be using.
            DefaultSchedule sched;
            if (schedule == null) {
                sched = new DefaultSchedule(schedulerPlugin);
            } else {
                sched = new DefaultSchedule(schedulerPlugin, schedule);
            }

            //Since we checked above that we could add this section, go ahead and add it.
            sched.addSection(section);

            //Since these are extra courses we are adding, if the rest of the courses dont fit, oh well, we
            //still want to see this schedule, so we add it to our list of valid schedules
            schedules.add(sched);

            if (!last) {
                buildExtraSchedules(schedules, sched, extra, newindex);
            }
        }
        if (!last) buildExtraSchedules(schedules, schedule, extra, newindex);
    }

    /**
     * Flag that we need to generate the schedules
     */
    private synchronized void setNeedsGenerating() {
        needsGenerating = true;
    }

    /**
     * Sort our possibleScchedules using the comparitor passed
     * @param comp the Comparitor to use in the sorting pocess
     */
    public synchronized void sortBy(Comparator<? super Schedule> comp) {
        lastComparator = comp;
        Collections.sort(possibleSchedules, comp);
    }

    /**
     * Returns the schedules that have been generated with {@link
     * #generateSchedules}, sorted in the order set by the {@code sortBy*}
     * methods.
     * @return the schedules that have been generated, sorted
     */
    public synchronized List<Schedule> getGeneratedSchedules() { return possibleSchedules; }

    /**
     * Returns the {@code Comparator} being used to sort the schedules.
     * @return the {@code Comparator} being used to sort the schedules
     */
    public synchronized Comparator<? super Schedule> getSortMethod() { return lastComparator; }

    /**
     * Return if a course is extra or not.
     * @param course The CourseDescriptor we are checking for extra status
     * @return True if the course exists and is indeed extra, false otherwise
     */
    public synchronized boolean isExtra(CourseDescriptor course) {
        SelectedCourse selectedCourse = selectedCourses.get(course);
        return selectedCourse != null && selectedCourse.isExtra();
    }

    /**
     * Returns a list of CourseDescriptors roughly sorted by department
     * @return list of CourseDescriptors roughly sorted by department
     */
    public synchronized List<CourseDescriptor> getAllCourses() {
        //@todo: does this limit us to 1000 courses?
        List<CourseDescriptor> courseList = new ArrayList<CourseDescriptor>(1000);
        Collection<Department> depts = schedulerData.getDepartments();
        //Loop through by department
        for (Department dept : depts) {
            //Add all the courses for our current department
            courseList.addAll(CourseDescriptor.getDescriptors(dept,
                    dept.getCourses()));
        }
        return courseList;
    }

    /**
     * This function takes a collection of courses, clears the selectedCourses variable and
     * repopulates it with the passed courses.  Finally, we set call setNeedsGenerating() so
     * the schedules get regenerated with the new courses.
     * @param courses A Collection of courses to make the selected courses
     */
    public synchronized void setSelectedCourses(Collection<SelectedCourse> courses) {
        selectedCourses.clear();
        for (SelectedCourse course : courses) addCourse(course);
        setNeedsGenerating();
    }

    /**
     * This adds a SelectedCourse to the selectedCourses list.
     * Also, we set call setNeedsGenerating() so the schedules get regenerated with the new courses.
     *
     * @param course SelectedCourse to add to our list of selectedCourses
     */
    public synchronized void addCourse(SelectedCourse course) {
        if (selectedCourses.put(course.getCourse(), course) == null) {
            setNeedsGenerating();
        }
    }

    /**
     * Remove a course from the selectedCourses list and set the flag for regenerating the schedule.
     * @param course CourseDescriptor to be removed from the list.
     */
    public synchronized void removeCourse(CourseDescriptor course) {
        if (selectedCourses.remove(course) != null) setNeedsGenerating();
    }

    /**
     * Remove all courses from the selectedCourses list.
     */
    public synchronized void removeAllCourses() {
        if (!selectedCourses.isEmpty()) {
            selectedCourses.clear();
            setNeedsGenerating();
        }
    }

    /**
     * Remove a collection of courses from the list of selected courses.
     * @param courses Collection<CourseDescriptor> to remove from the selctedCourses list.
     */
    public synchronized void removeCourses(Collection<CourseDescriptor> courses) {
        for (CourseDescriptor cd : courses) removeCourse(cd);
    }

    /**
     * Check if two courses completely conflict with eachother. Courses complete conflict if
     * there is no combo of sections that do not have a time conflict with eachother.
     * @param a CourseDescriptor first course to check
     * @param b CourseDescriptor second course to check
     * @return False if there are two sections that do not conflict, true otherwise
     */
    public boolean coursesConflict(CourseDescriptor a, CourseDescriptor b) {
        Collection<Section> sections1 = a.getActualCourse().getSections();
        Collection<Section> sections2 = b.getActualCourse().getSections();
        //If either course has no sections, they cant conflict with eachother, so return false.
        if (sections1.isEmpty() || sections2.isEmpty()) return false;
        //Loop through every combo of sections
        for (Section section : sections1) {
            for (Section section2 : sections2) {
                //Check each combo looking for sections that do not conflict with eachother
                if (!sectionsConflict(section, section2)) {
                    // we found two sections of these courses that don't conflict!
                    return false;
                }
            }
        }
        //Otherwise we got through all combos without finding two that dont conflict, so return true
        return true;
    }

    /**
     * Check if two sections conflict with eachother.
     * @param section1 The first section to check
     * @param section2 The second section to check
     * @return True if the two sections cannot be taken at the same time.
     */
    public boolean sectionsConflict(Section section1, Section section2) {
        return !section1.getYearMask().fitsInto(section2.getYearMask());
    }

    /**
     * Returns the value of the working variable.
     * @return Boolean value of working
     */
    public boolean isWorking() {
        return working;
    }

	/**
	 * Generates a collection of section descriptors that are used in at least one schedule
	 * 
	 * @return Collection of SectionDescriptors with each one being used at least once in a valid schedule
	 */
	public Collection<SectionDescriptor> getUsedSections(){
		Collection<SectionDescriptor> out = new LinkedList<SectionDescriptor>();
		for(Iterator<Schedule> i = possibleSchedules.iterator(); i.hasNext(); ){
			for(Iterator<UniqueSection> in = i.next().getSections().iterator(); in.hasNext(); ){
				out.addAll(in.next().getSectionDescriptors());
			}
		}
		return out;
	}
}
