/*
 *  Copyright (c) 2004, The University Scheduler Project
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of the University Scheduler Project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

package edu.rpi.scheduler.schedb;

import edu.rpi.scheduler.schedb.spec.*;

/**
 * This is one implementation of storing {@code Course} Data.<br>
 * Each AbstractCourse has or can have a:
 * <ul>
 * <li>Name</li>
 * <li>Course Number (IE: 101, not to be confused with CourseID)</li>
 * <li>Grade Type</li>
 * <li>Credits</li>
 * <li>Notes</li>
 * </ul>
 * 
 * This abstraction is extened in {@code DefaultCourse} which supports
 * sections.
 */
public abstract class AbstractCourse implements Course {
    private final String name;
    private final CourseID coursenum;
    private final GradeType gradeType;
    private final DoubleRange credits;
    private final Notes notes;


    /**
     * Constructor
     * @param coursename String with the name of course
     * @param coursenum CourseID for this course
     * @param gradeType GradeType obj for this course
     * @param credits IntRange obj for this course's credit's
     * @param notes Notes object for notes on this course

     */
    protected AbstractCourse(String coursename, CourseID coursenum,
                             GradeType gradeType, DoubleRange credits, Notes notes) {
        this.name = coursename;
        this.coursenum = coursenum;
        this.gradeType = gradeType;
        this.credits = credits;
        this.notes = notes;

    }

    /**
     * @return String containing the name of the course
     */
    public String getName() { return name; }

    public CourseID getNumber() { return coursenum; }
    
    public GradeType getGradeType() { return gradeType; }

    public DoubleRange getCredits() { return credits; }
    
    public Notes getNotes() { return notes; }
    /**
     * Compares course numbers between this course and 
     * the passed course object
     * @param o Course object to compare against
     */
    public int compareTo(Course o) {
        return getNumber().compareTo(o.getNumber());
    }

    /**
     * Returns a string of the format:
     * "Course 1234 : Intro to Bio (4 sections)"
     * for this course
     */
    public String toString() {
        return "Course " + getNumber() + ": " + getName()
                + " (" + getSections().size() + " sections)";
    }
}
