/*
 *  Copyright (c) 2004, The University Scheduler Project
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of the University Scheduler Project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

package edu.rpi.scheduler.schedb;

import edu.rpi.scheduler.schedb.spec.Department;
import edu.rpi.scheduler.schedb.spec.Course;
import edu.rpi.scheduler.DefensiveTools;

import java.util.List;
import java.util.ArrayList;
import java.util.Collection;

/**
 * Description of a particular course.
 * The main data here is a link between department and course.
 * I believe this is so one course can have two descriptors if it is 
 * offered through two seperate departments.
 */
public class CourseDescriptor implements Comparable<CourseDescriptor> {
    public static List<CourseDescriptor> getDescriptors(Department dept,
            Collection<Course> courses) {
        List<CourseDescriptor> cds = new ArrayList<CourseDescriptor>();
        for (Course course : courses) {
            cds.add(new CourseDescriptor(dept, course));
        }
        return cds;
    }
    /**Department object for this course's department*/
    private final Department dept;
    /**Course object for this description*/
    private final Course course;

    /**
     * Constructor: Create a CourseDescriptor from the 
     * department and course information contained in a 
     * CourseDescriptor
     * @param other SectionDescriptor to pull Course info from 
     */
    private CourseDescriptor(SectionDescriptor other) {
        this(other.getDept(), other.getCourse());
    }
    /**
     * Constructor: Create a CourseDescriptor from the 
     * department and actual course information contained in another
     * CourseDescriptor 
     * @param other CourseDescriptor to pull Course info from 
     */
    public CourseDescriptor(CourseDescriptor other) {
        this(other.getDept(), other.getActualCourse());
    }

    /**
     * Constructor: Create a CourseDescriptor from raw deptartment
     * and course objects
     * Throws an error if either parameter is null
     * @param dept Department to use for this CourseDescriptor
     * @param course Course to use for this CourseDescriptor
     */
    public CourseDescriptor(Department dept, Course course) {
        DefensiveTools.checkNull(dept, "dept");
        DefensiveTools.checkNull(course, "course");

        this.dept = dept;
        this.course = course;
    }
    /**
     * Return the department for this course
     * @return Department object this course belongs to.
     */
    public Department getDept() { return dept; }
    
    /**
     * Return the actual Course object this discriptor is describing
     * @return The actual Course object this discriptor is describing
     */
    public Course getActualCourse() { return course; }

    /**
     * Compares this CourseDescriptor with another CourseDesciptor obj.
     * First compares departments and then compare courses
     */
    public int compareTo(CourseDescriptor sd) {
        int deptc = getDept().compareTo(sd.getDept());
        if (deptc != 0) return deptc;

        return getActualCourse().compareTo(sd.getActualCourse());
    }

    /**compares two CourseDescriptors (returns false if object passed 
     * is not of type CourseDescriptor)
     * @param o Object to compare against
     * @return boolean, True if departments are equal and actual courses are equal
     * False otherwise 
     */
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof CourseDescriptor)) return false;

        CourseDescriptor othersd = (CourseDescriptor) o;

        if (!getDept().equals(othersd.getDept())) return false;
        if (!getActualCourse().equals(othersd.getActualCourse())) return false;

        return true;
    }

    /**
     * Creates and returns a hashCode for this CourseDescriptor 
     * based on the hash codes for the department and Actual Course
     */
    public int hashCode() {
        int result;
        result = getDept().hashCode();
        result = 29 * result + getActualCourse().hashCode();
        return result;
    }
    /**
     * Returns a string with the department abreviation, course number, and 
     * course name in that order.
     * @return String with CourseDescriptor information
     */
    public String toString() {
        return getDept().getAbbrev() + " " + getActualCourse().getNumber()
                + ": " + getActualCourse().getName();
    }
    /**
     * Creates and returns a new CourseDescriptor object for a particular
     * section of a course given by the passed SectionDescriptor
     * @param other the SectionDescriptor to use to generate a new CourseDescriptor 
     * @return CourseDescriptor object for the given section
     */
    public static CourseDescriptor getInstanceForSection(SectionDescriptor other) {
        return new CourseDescriptor(other);
    }
}
