/*
 *  Copyright (c) 2004, The University Scheduler Project
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of the University Scheduler Project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

package edu.rpi.scheduler.schedb;

import edu.rpi.scheduler.schedb.spec.*;

import java.util.*;

/**
 * This class is a representation of a single user's (partly)constructed
 * schedule.  
 */
public class DefaultSchedule implements Schedule {
    private final SchedulerDataPlugin plugin;

    private final List<UniqueSection> sections = new ArrayList<UniqueSection>();
    private YearMask<?> mask = null;
	private boolean saveMemory = false;
	//Was originally here so it would be cached.  To Optimize for memory let's just build it
	//each time.
	//private YearMask<?> classMask = null;

	private int daysOff = -1;
    private int[] times = null;

    /**
     * Constructor given a SchedulerDataPlugin
     * @param plugin SchedulerDataPlugin to use to construct this object
     */
    public DefaultSchedule(SchedulerDataPlugin plugin) {
        this(plugin, null);
    }

    /**
     * Constructor given a SchedulerDataPlugin and Schedule object
     * @param schedulerPlugin
     * @param schedule
     */
    public DefaultSchedule(SchedulerDataPlugin schedulerPlugin, Schedule schedule) {
        plugin = schedulerPlugin;

		saveMemory = System.getProperty("saveMemory").equals("true");
		if(!saveMemory){
			mask = plugin.getTimeRepresentation().newYearMask();
		}

        if (schedule != null) {
            sections.addAll(schedule.getSections());
			if(!saveMemory){
				mask.merge(schedule.getTimeMask());
			}
		}
    }

    /**
     * Returns the SchedulerDataPlugin for this Schedule
     * @return the SchedulerDataPlugin for this Schedule
     */
    public SchedulerDataPlugin getSchedulerPlugin() { return plugin; }

    public void addSection(UniqueSection section) {
        sections.add(section);
		if(!saveMemory){
			mask.merge(section.getTimeMask());
		}
		reset();
    }

    public boolean canAdd(UniqueSection section) {
        YearMask<?> mask = section.getTimeMask();
        return canAdd(mask);
    }

    public boolean canAdd(YearMask<?> mask) {
		if(!saveMemory){
			return mask.fitsInto(this.mask);
		}else{
			YearMask<?> thisMask = plugin.getTimeRepresentation().newYearMask();
			for(UniqueSection us:sections){
				thisMask.merge(us.getTimeMask());
			}
			return mask.fitsInto(thisMask);
		}
	}

    public YearMask<?> getTimeMask() {
		if(!saveMemory){
			return mask;
		}else{
			YearMask<?> thisMask = plugin.getTimeRepresentation().newYearMask();
			for(UniqueSection us:sections){
				thisMask.merge(us.getTimeMask());
			}
			return thisMask;
		}
	}

    public List<UniqueSection> getSections() {
        return sections;
    }

	public List<UniqueSection> getSections(String Term, String PartOfTerm){
		List<UniqueSection> outGoing = new LinkedList<UniqueSection>();
		for(UniqueSection s: sections){
			boolean termMatch = (s.getTimeMask().getActiveTerms().contains(Term));
			boolean PoTMatch = (s.getTimeMask().getTerm(Term).getActivePartOfTerms().contains(PartOfTerm));
			if(termMatch && PoTMatch){
				outGoing.add(s);
			}
		}
		return outGoing;
	}

	/**
     * Reset this schedule back to 
     * daysOff = -1
     * times = null
     * classMask = null
     */
    private void reset() {
        daysOff = -1;
        times = null;
//        betweens = null;
        //classMask = null;
    }

    public YearMask<?> getClassMask() {
        //if (classMask == null) {
            YearMask<?> mask = plugin.getTimeRepresentation().newYearMask();

            for (UniqueSection section : sections) {
                mask.merge(section.getTimeMask());
            }

        //    classMask = mask;
        //}

        //return classMask;

		return mask;
	}

    //TODO: make sure this is behaving how we want it to with multiple terms
	/**
	 * This totals the number of days of classes we have over the entire year
	 * @return int equal ot the number of days of classes we have over the course of the year
	 */
	public int getDaysOfClass() {
        if (daysOff == -1) {
            int days = 0;
            YearMask<?> mask = getClassMask();

            //This loops over each term
            for (int i = 0; i < Year.getInstance().getNumberOfTerms(); i++) {
                //This loops over each part of term
                for (int j = 0; j < Year.getInstance().getTerm(i).getNumberOfPartOfTerms(); j++) {
                    //This loops over each day of the week
                    for (int k = 0; k < 7; k++) {
                        if (!mask.getTerm(i).getPartOfTerm(j).getDayMask(k).isEmpty()) days++;
                    }
                }
            }

            daysOff = days;
        }

        return daysOff;
    }

    //TODO: make sure this is behaving how we want it to with multiple terms
	/**
	 * This function gets the sum of the amount of class time for each day and returns it in an array
	 * @return array of ints one for each day, up through the terms and part of terms
	 */
	public int[] getTimeSums() {
    	//if times is null we have to calculate it for the first time
        if (times == null) {

			int[] halves = new int[Year.getInstance().getNumberOfPartOfTerms()*7];
            YearMask<?> mask = getClassMask();
            
            //This loops over each term
            for (int i = 0; i < Year.getInstance().getNumberOfTerms(); i++) {
                //This loops over each part of term
                for (int j = 0; j < Year.getInstance().getTerm(i).getNumberOfPartOfTerms(); j++) {
                    //This loops over each day of the week
                    for (int k = 0; k < 7; k++) {
                        DayMask day = mask.getTerm(i).getPartOfTerm(j).getDayMask(k);

                        int time = day.getTimeBlockSum();

                        halves[i+j+k] = time;
                    }
                }
            }

            Arrays.sort(halves);
            // reverse the sorting
            for (int i = 0; i < halves.length/2; i++) {
                int j = halves.length-i-1;
                int temp = halves[i];
                halves[i] = halves[j];
                halves[j] = temp;
            }
            times = halves;
        }

        return times;
    }

    //TODO: make sure this is behaving how we want it to with multiple terms
	/**
	 * This function calculates the time inbetween classes for a given term/part of term
	 * @param Term The term identifier for the term we are working on
	 * @param PartOfTerm the part of term identifier for the PoT we are working on
	 * @return List of list of time periods representing the time between classes
	 */
	public List<List<TimePeriod>> getBetweens(String Term, String PartOfTerm) {
        List<List<TimePeriod>> periodsByDay = new ArrayList<List<TimePeriod>>(7);
        for (int i = 0; i < 7; i++) {
            periodsByDay.add(new ArrayList<TimePeriod>());
        }
        for (UniqueSection section : sections) {
			Collection<SectionDescriptor> sectionDescriptors = section.getSectionDescriptors();
            // we only need to look at the first section descriptor, since the
            // rest are part of the same unique section, so they have the same
            // class periods
            SectionDescriptor sd = sectionDescriptors.iterator().next();

			//make sure that we only check sections that are in our Term / PartOfTerm
			if(sd.getActualSection().getTerm().equals(Term) && sd.getActualSection().getPartOfTerm().equals(PartOfTerm)){
				for (ClassPeriod period : sd.getActualSection().getPeriods()) {
					DailyTimePeriod tp = period.getTimePeriod();
					int day = 0;
					for (boolean on : tp.getDays()) {
						if (on) periodsByDay.get(day).add(tp.getPeriod());
						day++;
					}
				}
			}
		}

        List<List<TimePeriod>> betweens = new ArrayList<List<TimePeriod>>(7);
        for (int i = 0; i < 7; i++) {
            betweens.add(new ArrayList<TimePeriod>(5));
        }
        int d = 0;
        for (List<TimePeriod> periods : periodsByDay) {
            Collections.sort(periods);
            for (int i = 1; i < periods.size(); i++) {
                TimePeriod first = periods.get(i-1);
                TimePeriod second = periods.get(i);
                betweens.get(d).add(TimePeriod.getPeriodBetween(first, second));
            }
            d++;
        }
        return betweens;
    }
}
