/*
 *  Copyright (c) 2004, The University Scheduler Project
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of the University Scheduler Project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

package edu.rpi.scheduler.schedb;

import edu.rpi.scheduler.schedb.spec.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Represents a set of scheduler data needed for a SchedulerEngine Class
 * Has a list of department, courses, and sections for this schedule.
 */
public class DefaultSchedulerData implements SchedulerData {
    private Date lastModified = null;
    private String dataSetId = null;

    private List<Department> depts = new ArrayList<Department>();
    private List<Course> courses = new ArrayList<Course>();
    private List<Section> sections = new ArrayList<Section>();

    private final DataContext context;


    /**
     * Consructor that takes a DataContext to get things started
     * @param context The context we are getting our data from.
     */
    public DefaultSchedulerData(DataContext context) {
        this.context = context;
    }

    public DataContext getDataContext() {
        return context;
    }

    public String getDataSetId() {
        return dataSetId;
    }

    /**
     * Set the ID of this data set with the given string
     * @param dataSetId String that the dataSetID is supposto be set to
     */
    public void setDataSetId(String dataSetId) {
        this.dataSetId = dataSetId;
    }

    /**
     * Add a complete department's courses to the schedule.
     * This adds an entire department's courses, and sections
     * @param dept Department object that has the courses and sections
     * that we want to add.
     */
    public void addCompletedDepartment(Department dept) {
        depts.add(dept);
        for (Course cours : dept.getCourses()) {
            courses.add(cours);

            for (Section section : cours.getSections()) {
                sections.add(section);
            }
        }
    }

    public Date getLastModified() {
        return lastModified;
    }

    /**
     * Set the lastModified date to the given Date object 
     * @param lastModified Date object that represents the last modified date
     */
    public void setLastModified(Date lastModified) {
        this.lastModified = lastModified;
    }

    public List<Department> getDepartments() {
        return depts;
    }

    public List<Course> getCourses() {
        return courses;
    }

    public List<Section> getSections() {
        return sections;
    }

    public Department getDepartment(String abbrev) {
        for (Department dept : depts) {
            if (dept.getAbbrev().equals(abbrev)) return dept;
        }
        return null;
    }

    //public List<Term> getTerms(){
     //   return terms;
    //}
}
