/*
 *  Copyright (c) 2004, The University Scheduler Project
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of the University Scheduler Project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

package edu.rpi.scheduler.schedb;

import edu.rpi.scheduler.schedb.spec.*;

import java.util.Collection;

/**
 * This is an implementation of {@code Section} that stores 
 * lots of information about the section.
 */
public class DefaultSection implements Section {
    private final SchedulerDataPlugin plugin;
    private final SectionNumber number;
    private final SectionID secid;
    private final Collection<ClassPeriod> periods;
    private final Notes notes;
    private final int seats;
    private YearMask<?> yearMask = null;
    private final String term;
    private final String partofterm;

    /**
     * DefaultSection constructor 
     * Simply stores all data for retrival later
     * 
     * @param plugin Plugin for the SchedulerData
     * @param number SectionNumber for this section
     * @param secid SectionID for this section
     * @param seats int of the number of seats in this section
     * @param periods A collection of ClassPeriod objects representing the periods this section meets
     * @param notes Notes object for notes for this section
     * @param partofterm PartOfTerm String for the part of term for this section
     */
    public DefaultSection(SchedulerDataPlugin plugin, SectionNumber number, SectionID secid,
            int seats, Collection<ClassPeriod> periods, Notes notes, String partofterm, String term) {
        this.plugin = plugin;
        this.number = number;
        this.secid = secid;
        this.seats = seats;
        this.periods = periods;
        this.notes = notes;
        this.term = term;
        this.partofterm = partofterm;
    }

    public SectionNumber getNumber() { return number; }

    public SectionID getID() { return secid; }

    public int getSeats() { return seats; }

    public Collection<ClassPeriod> getPeriods() { return periods; }

    /**
     * NOTE: returned object is mutable
     */
    public YearMask<?> getYearMask() {
        if (yearMask == null) {
            TimeRepresentation timeRep = plugin.getTimeRepresentation();
            yearMask = timeRep.newYearMask();
            for (ClassPeriod period : periods) {
				DailyTimePeriod timeP = period.getTimePeriod();
				YearMask time = timeRep.getYearMask(timeP);
				yearMask.merge(time);
				int i = 0;
			}
        }

        return yearMask;
    }


    public String getTerm() { return term; }

    public String getPartOfTerm() { return partofterm; }

    public Notes getNotes() { return notes; }

    public int compareTo(Section o) {
        return getNumber().compareTo(o.getNumber());
    }

    public String toString() {
        return getID() + ": section " + getNumber();
    }
}
