package edu.rpi.scheduler.schedb;

import edu.rpi.scheduler.DefensiveTools;

import java.util.regex.Pattern;
import java.util.regex.Matcher;

/**
 * A representation of a range of Doubles (used to represent possible
 * Credits).  Consists of a starting and ending value.
 */
public class DoubleRange {
    private static final Pattern rangeRE = Pattern.compile("(\\d+)(?:-(\\d+))?");

    /**
     * Takes a string in and parses it to a DoubleRange (either one with a start
     * and finish or a zero length range)
     * @param str The string to be parsed
     * @return DoubleRange object constructed from the string data
     */
    public static final DoubleRange parseRange(String str) {
        Matcher m = rangeRE.matcher(str);
        if (!m.matches()) return null;

        Double from = Double.parseDouble(m.group(1));
        String tostr = m.group(2);
        if (tostr == null) return new DoubleRange(from);
        else return new DoubleRange(from, Double.parseDouble(tostr));
    }

    private final Double from;
    private final Double to;

    /**
     * Construct a range to and from the same value
     * @param val the value for the start and end of the range
     */
    public DoubleRange(Double val) {
        this(val, val);
    }

    /**
     * Construct a range with a start and stop
     * @param from Lower limit of the range
     * @param to Upper limit of the range
     * @throws IllegalArgumentException if from > to
     */
    public DoubleRange(Double from, Double to) {
        DefensiveTools.checkRange(from, "from", 0);

        if (from > to) {
            throw new IllegalArgumentException("invalid range " + from
                    + " to " + to);
        }

        this.from = from;
        this.to = to;
    }

    /**
     * Determines if the current range is truely a range
     * @return true if 'to != from' false of 'to == from'
     */
    public boolean isRange() {
        return !to.equals(from);
    }

    /** @return lower bound of the range */
    public Double getFrom() {
        return from;
    }
    /** @return upper bound of the range */
    public Double getTo() {
        return to;
    }
    /**
     * Returns a string representation of the range in the format:
     * "from - to" or just "from"
     * @return a String representation of the current range
     */
    public String toString() {
        return "" + from + (isRange() ? "-" + to : "");
    }
}
