/*
 *  Copyright (c) 2004, The University Scheduler Project
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of the University Scheduler Project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

package edu.rpi.scheduler.schedb;

import edu.rpi.scheduler.schedb.spec.Section;
import edu.rpi.scheduler.schedb.spec.Course;
import edu.rpi.scheduler.schedb.spec.Department;
import edu.rpi.scheduler.DefensiveTools;

import java.util.List;
import java.util.ArrayList;
import java.util.Collection;

/**
 * Description of a particular section of a particular course
 * This links together department, course, and section information.
 * This is so that sections can be linked to different departments 
 * and potentially courses.
 */
public class SectionDescriptor implements Comparable<SectionDescriptor> {
    public static List<SectionDescriptor> getDescriptors(Department dept, Course course,
            Collection<Section> sections) {
        List<SectionDescriptor> sds = new ArrayList<SectionDescriptor>();
        for (Section section : sections) {
            sds.add(new SectionDescriptor(dept, course, section));
        }
        return sds;
    }

    public static List<SectionDescriptor> getDescriptors(CourseDescriptor course,
            Collection<Section> sections) {
        return getDescriptors(course.getDept(), course.getActualCourse(), sections);
    }

    private final Department dept;
    private final Course course;
    private final Section section;

    /**
     * Copy Constructor
     * @param other SectionDescription to copy
     */
    public SectionDescriptor(SectionDescriptor other) {
        this(other.getDept(), other.getCourse(), other.getActualSection());
    }
    /**
     * Constructor given a CourseDescriptor and a Section
     * The department and ActualCourse information is simply pulled from
     * the CourseDescriptor and added to the new SectionDescriptor
     * @param other The CourseDescriptor to use as a start for this section
     * @param section the particular Section object that we are going to Describe
     */
    public SectionDescriptor(CourseDescriptor other, Section section) {
        this(other.getDept(), other.getActualCourse(), section);
    }

    /**
     * Raw SectionDescriptor constructor, explicitly given all the info needed
     * None of the arguments can be null or an error will be thrown.
     * @param dept Department object for the described course
     * @param course Course object for the described course
     * @param section Section object for the described course
     */
    public SectionDescriptor(Department dept, Course course, Section section) {
        DefensiveTools.checkNull(dept, "dept");
        DefensiveTools.checkNull(course, "course");
        DefensiveTools.checkNull(section, "section");

        this.dept = dept;
        this.course = course;
        this.section = section;
    }
    /**@return Department object for this description of a course*/
    public Department getDept() { return dept; }
    /**@return Course object for this description of a course*/
    public Course getCourse() { return course; }
    /**@return Section object for this description of a course*/
    public Section getActualSection() { return section; }

    /**
     * Compares this CourseDescriptor with another given CourseDescriptor
     * Compares department, course and section (in that order)
     */
    public int compareTo(SectionDescriptor sd) {
        int deptc = getDept().compareTo(sd.getDept());
        if (deptc != 0) return deptc;

        int sc = getCourse().compareTo(sd.getCourse());
        if (sc != 0) return sc;

        return getActualSection().compareTo(sd.getActualSection());
    }

    /**
     * Equals function
     * @return false if parameter is not of type SectionDescriptor,
     * or if department, course, and section do not match.
     * Returns true otherwise
     */
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof SectionDescriptor)) return false;

        SectionDescriptor othersd = (SectionDescriptor) o;

        if (!getDept().equals(othersd.getDept())) return false;
        if (!getCourse().equals(othersd.getCourse())) return false;
        if (!getActualSection().equals(othersd.getActualSection())) return false;

        return true;
    }
    /**
     * Generates a unique hash code for this SectionDescriptor
     * @return int a unique hash code for this SectionDescriptor
     */
    public int hashCode() {
        int result = getDept().hashCode();
        result = 29 * result + getCourse().hashCode();
        result = 29 * result + getActualSection().hashCode();
        return result;
    }
    /**
     * Returns a string with the department, course number, course name,
     * "sec#" and section number.
     * @return a string with the department, course number, course name,
     * "sec#" and section number. 
     */
    public String toString() {
        return getDept().getName() + " " + getCourse().getNumber()
                + ": " + getCourse().getName() + " sec#" + getActualSection().getNumber();
    }
}
