/*
 *  Copyright (c) 2004, The University Scheduler Project
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of the University Scheduler Project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

package edu.rpi.scheduler.schedb;

import edu.rpi.scheduler.schedb.spec.Year;
import edu.rpi.scheduler.schedb.spec.DayMask;

import java.util.ArrayList;
import java.util.List;
import java.util.Collection;
import java.util.LinkedList;

/**
 * This class is a mask that has several week masks (one for each Part Of Term)
 */
public class TermMask<WM extends WeekMask> {

	private String identifier; 
	public static <WM extends WeekMask> TermMask<WM> newMask(Class<WM> cl, String identifier) {
        try {
            return new TermMask<WM>(cl, identifier);
        } catch (IllegalAccessException e) {
            throw new IllegalArgumentException("cannot create new "
                    + cl.getName(), e);
        } catch (InstantiationException e) {
            throw new IllegalArgumentException("cannot create new "
                    + cl.getName(), e);
        }
    }

    private List<WM> weeks;

    private TermMask(Class<WM> cl, String identifier) throws IllegalAccessException,
            InstantiationException {
        List<WM> weeks = new ArrayList<WM>(Year.getInstance().getTerm(identifier).getNumberOfPartOfTerms());
        for (int i = 0; i < Year.getInstance().getTerm(identifier).getNumberOfPartOfTerms(); i++) {
			//TODO: I shouldnt have to cast this.
			weeks.add((WM) WeekMask.newMask(BitSetDayMask.class));
        }
        this.weeks = weeks;
		this.identifier = identifier;
	}

	public TermMask(TermMask<? extends WM> other, String identifier) {
        this(other.weeks, identifier);
	}

    public TermMask(Collection<? extends WM> mask, String identifier) {

        if (mask.size() != Year.getInstance().getTerm(identifier).getNumberOfPartOfTerms()) {
            throw new IllegalArgumentException("array is not of length " + Year.getInstance().getTerm(identifier).getNumberOfPartOfTerms() +
                    " (it's " + mask.size() + ")");
        }
        this.weeks = new ArrayList<WM>(mask);
        assert weeks.size() == Year.getInstance().getTerm(identifier).getNumberOfPartOfTerms();
		this.identifier = identifier;
	}

	public final boolean isOn(int Week, int Day, int block) {
        return weeks.get(Week).isOn(Day, block);
    }

    public final boolean isEmpty() {
        for (WeekMask mask : weeks) {
			if (mask == null) return true;
			if (!mask.isEmpty()) return false;
        }
        return true;
    }

    public final boolean isEmpty(int Week) {
        return weeks.get(Week).isEmpty();
    }

    public final boolean add(int Week, int Day, int block) {
        return weeks.get(Week).add(Day, block);
    }

    public final WM add(int Week, WeekMask weekMask) {
        return weeks.set(Week, (WM) weekMask);
    }

    public final boolean delete(int Week, int Day, int block) {
        return weeks.get(Week).delete(Day, block);
    }

    public final boolean fill(int Week) {
        return weeks.get(Week).fill();
    }
    public final boolean fill(){
        boolean success = true;
        int i =0;
        for (WM w : weeks) {
            success = success && weeks.get(i).fill();
            i++;
        }
        return success;
    }
    public final boolean clear(int Week) {
        return weeks.get(Week).clear();
    }
    public final boolean clear(){
        boolean success = true;
        int i =0;
        for (WM w : weeks) {
            success = success && weeks.get(i).clear();
            i++;
        }
        return success;
    }
    public final boolean fitsInto(TermMask<?> other) {

        int i =0;
        for (WM w : weeks) {
            if (!weeks.get(i).fitsInto(other.getWeekMask(i))) return false;
            i++;
        }

        return true;
    }

    public final WeekMask getPartOfTerm(int week){
        return weeks.get(week);
    }

	/**
	 * This returns what PartOfTerms have some time masked out in them
	 * @return list of Identifier Strings
	 */
	public List<String> getActivePartOfTerms(){
		List<String> output = new LinkedList<String>();
		int i=0;
		for(WeekMask w: weeks){
			if(!weeks.get(i).isEmpty()){
				output.add(Year.getInstance().getTerm(this.identifier).getPartOfTerm(i).getIdentifier());
			}
			i++;
		}
		return output;
    }
	public final void merge(TermMask<?> other) {
         int i =0;
        for (WM w : weeks) {
            weeks.get(i).merge(other.getWeekMask(i));
            i++;
        }
    }

    public WM getWeekMask(int Week) {
        return weeks.get(Week);
    }

    public int getTimeBlockSum() {
        int time = 0;
        for (WeekMask Week : weeks) time += Week.getTimeBlockSum();

        return time;
    }

    public final boolean equals(Object other) {
        if (!(other instanceof TermMask)) return false;

        return weeks.equals(((TermMask<?>) other).weeks);
    }

    public int hashCode() {
        int code = 0;
        for (WeekMask mask : weeks) code ^= mask.hashCode();
        return code;
    }

	public String toString(){
		if(isEmpty()) return "";
		return "Populated Term";
	}
}
