/*
 *  Copyright (c) 2004, The University Scheduler Project
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of the University Scheduler Project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

package edu.rpi.scheduler.schedb;

import edu.rpi.scheduler.schedb.spec.Year;
import edu.rpi.scheduler.schedb.spec.SchedulerDataPlugin;
import edu.rpi.scheduler.schedb.spec.DailyTimePeriod;
import edu.rpi.scheduler.schedb.spec.Term;

import java.util.ArrayList;
import java.util.List;
import java.util.Collection;
import java.util.LinkedList;

/**
 * This class is a mask that consists of several term masks
 */
public class YearMask<TM extends TermMask> {
    public static <TM extends TermMask> YearMask<TM> newMask(Class<TM> cl) {
        try {
            return new YearMask<TM>(cl);
        } catch (IllegalAccessException e) {
            throw new IllegalArgumentException("cannot create new "
                    + cl.getName(), e);
        } catch (InstantiationException e) {
            throw new IllegalArgumentException("cannot create new "
                    + cl.getName(), e);
        }
    }

    private final List<TM> Terms;

    private YearMask(Class<TM> cl) throws IllegalAccessException,
            InstantiationException {
        List<TM> Terms = new ArrayList<TM>(Year.getInstance().getNumberOfTerms());
        for (int i = 0; i < Year.getInstance().getNumberOfTerms(); i++) {
			//TODO: I shouldnt have to cast this...
			Terms.add((TM) TermMask.newMask(WeekMask.class, Year.getInstance().getTerm(i).getIdentifier()));
        }
        this.Terms = Terms;
    }

    private YearMask(YearMask<? extends TM> other) {
        this(other.Terms);
    }

    /*
    This method is BAD.... we should use the newMask function instead!
    public YearMask(){
        this.Terms = new ArrayList<TM>(Year.getInstance().getNumberOfTerms());
		for(int i = 0; i < Year.getInstance().getInstance().getNumberOfTerms(); i++){
			Terms.add(cl.newInstance());
		}
	}*/

    public YearMask(Collection<? extends TM> mask) {
        if (mask.size() != Year.getInstance().getNumberOfTerms()) {
            throw new IllegalArgumentException("array is not of proper length " +
                    "(it's " + mask.size() + ")");
        }
        this.Terms = new ArrayList<TM>(mask);

        assert Terms.size() == Year.getInstance().getNumberOfTerms();
    }
	
	public final boolean isOn(int Term, int Week, int Day, int block) {
        return Terms.get(Term).isOn(Week, Day, block);
    }

    public final boolean isEmpty() {
        for (TermMask mask : Terms) {
            if (!mask.isEmpty()) return false;
        }
        return true;
    }

    public final boolean isEmpty(int Term) {
        return Terms.get(Term).isEmpty();
    }

    public final boolean add(int Term, int Week, int Day, int block) {
        return Terms.get(Term).add(Week, Day, block);
    }

    public final TM add(int Term, TermMask termMask) {
        return Terms.set(Term, (TM) termMask);
    }
    
    public final boolean delete(int Term, int Week, int Day, int block) {
        return Terms.get(Term).delete(Week, Day, block);
    }

    public final boolean fill(int Term) {
        return Terms.get(Term).fill();
    }

    public final boolean clear(int Term) {
        return Terms.get(Term).clear();
    }

    public final boolean fitsInto(YearMask<?> other) {
        int i =0;
        for (TM t : Terms) {
            if (!Terms.get(i).fitsInto(other.getTermMask(i))) return false;
            i++;
        }

        return true;
    }

    public final TermMask getTerm(int term){
        return Terms.get(term);
    }

	public final TermMask getTerm(String term){
		for(int i=0; i<Year.getInstance().getNumberOfTerms(); i++){
			if(Year.getInstance().getTerm(i).getIdentifier().equals(term)){
				return getTerm(i);
			}
		}
		return null;
	}

	public final void merge(YearMask<?> other) {
        int i =0;
        for (TM t : Terms) {
            Terms.get(i).merge(other.getTermMask(i));
            i++;
        }
    }

	/**
	 * This gets the terms in which this mask has blocked times
	 * @return List of Identifier Strings
	 */
	public List<String> getActiveTerms(){
		List<String> output = new LinkedList<String>();
		int i=0;
		for(TermMask t: Terms){
			if(!Terms.get(i).isEmpty()){
				output.add(Year.getInstance().getTerm(i).getIdentifier());
			}
			i++;
		}
		return output;//todo fix me!
    }

	public TM getTermMask(int Term) {
        return Terms.get(Term);
    }

    public int getTimeBlockSum() {
        int time = 0;
        for (TermMask Term : Terms) time += Term.getTimeBlockSum();

        return time;
    }

    public int getNumTerms(){
        return Terms.size();
    }

	public final boolean equals(Object other) {
        if (!(other instanceof YearMask)) return false;

        return Terms.equals(((YearMask<?>) other).Terms);
    }

    public int hashCode() {
        int code = 0;
        for (TermMask mask : Terms) code ^= mask.hashCode();
        return code;
    }

	public String toString(){
		if(isEmpty()) return "";
		else return "Populated Year";
	}
}
