/*
 *  Copyright (c) 2004, The University Scheduler Project
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of the University Scheduler Project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

package edu.rpi.scheduler.schedb.load;

import edu.rpi.scheduler.schedb.DefaultSection;
import edu.rpi.scheduler.schedb.StringSectionID;
import edu.rpi.scheduler.schedb.StringSectionNumber;
import edu.rpi.scheduler.schedb.load.spec.DataElement;
import edu.rpi.scheduler.schedb.load.spec.NotesLoader;
import edu.rpi.scheduler.schedb.load.spec.PeriodLoader;
import edu.rpi.scheduler.schedb.load.spec.SectionLoader;
import edu.rpi.scheduler.schedb.spec.*;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.Collection;

import static edu.rpi.scheduler.schedb.load.spec.DataObjectType.SECTION;

public class XmlSectionLoader extends XmlLoader implements SectionLoader {
    private static final Logger logger
            = Logger.getLogger(XmlSectionLoader.class.getPackage().getName());


    public XmlSectionLoader(MutableDataLoadingContext context) {
        super(context);
    }

    public Section loadSection(DataElement element)
            throws DbLoadException {
        XmlDataElement xel = (XmlDataElement) element;

        Element secel = xel.getDomElement();

        SectionNumber number = readNumber(secel);
        SectionID secid = readSecId(secel);

        String use = null;
        if (number != null) use = number.toString();
        else if (secid != null) use = secid.toString();
        if (use != null) {
            getContext().fireLoadingObject(SECTION, use);
        }

        String partofterm = readPartOfTerm(secel);
        String term = readTerm(secel);

        int seats = readSeats(secel);
        List<ClassPeriod> periods = readPeriods(secel, term, partofterm);
        Notes notes = readNotes(secel);
        DefaultSection section = createSection(number, secid, seats, periods,
                notes, partofterm, term);
        readCustomData(secel, section);

        return section;
    }

    protected void readCustomData(Element secel, Section section) {
    }

    private int readSeats(Element secel) throws DbLoadException {
        String numstr = getString(secel, "seats");
        return numstr == null ? -1 : Integer.parseInt(numstr);
    }

    /*
    <section crn="90589" number="2">
    <period type="lecture" professor="Barthel" days="tue,fri" starts="8:00AM"
    ends="9:20AM" />
    </section>
    */
    protected DefaultSection createSection(SectionNumber number,
            SectionID secid, int seats, List<ClassPeriod> periods, Notes notes, String partofterm, String term) {
        return new DefaultSection(getContext().getPlugin(), number, secid, seats, periods, notes, partofterm, term);
    }

    protected String getString(Element secel, String attrname)
            throws DbLoadException {
        if (!secel.hasAttribute(attrname)) return null;
        else return secel.getAttribute(attrname);
    }

    protected SectionNumber readNumber(Element coursel)
            throws DbLoadException {
        String numstr = getString(coursel, "number");
        return numstr == null
                ? null : new StringSectionNumber(numstr);

        /*
        Orignal (much better) code
         return numstr == null
                ? null : new IntSectionNumber(Integer.parseInt(numstr));
        */
    }

    protected SectionID readSecId(Element coursel) throws DbLoadException {
        String idstr = getString(coursel, "crn");
        return idstr == null ? null : new StringSectionID(idstr);
    }

	protected String readTerm(Element secel) throws DbLoadException {
        String term = getString(secel, "term");
        return term;
    }
	
	protected String readPartOfTerm(Element secel) throws DbLoadException {
        String partofterm = getString(secel, "part-of-term");
        return partofterm;
    }

	/**
	 * This function actually loops through and associates periods with this section.
	 *
	 * It gets it's period data from the XML by calling readPeriod.
	 * @param coursel the Element for this section
	 * @param Term the Term that this period is in (read from the section)
	 * @param PartOfTerm the PartOfTerm this period is in (or "Full Semester")
	 * @return List of ClassPeriods one for each period this class meets 
	 * @throws DbLoadException if something is wrong with the XML
	 */
	protected List<ClassPeriod> readPeriods(Element coursel, String Term, String PartOfTerm) throws DbLoadException {
        List<ClassPeriod> periods = new ArrayList<ClassPeriod>();
        NodeList perels = coursel.getElementsByTagName("period");
        for (int i = 0; i < perels.getLength(); i++) {
            Element perel = (Element) perels.item(i);
			//handle full semesters
			if(PartOfTerm.equals("Full Semester")){
				for(int j=0; j<Year.getInstance().getTerm(Term).getNumberOfPartOfTerms(); j++){
					try {
						periods.add(readPeriod(perel, Term, Year.getInstance().getTerm(Term).getPartOfTerm(j).getIdentifier()));
					} catch (DbLoadException e) {
						logger.log(Level.WARNING, "Error reading period", e);
						continue;
					}
				}
			}else{
				//This one is hit most of the time (non full semester)
				try {
					periods.add(readPeriod(perel, Term, PartOfTerm));
				} catch (DbLoadException e) {
					logger.log(Level.WARNING, "Error reading period", e);
					continue;
				}
        	}}
        return periods;
    }

    /*
    <period type="lecture" professor="Steigler" days="tue,fri" starts="2:00PM"
    ends="3:20PM" />

    */
    protected ClassPeriod readPeriod(Element perel, String Term, String PartOfTerm) throws DbLoadException {
        PeriodLoader loader = getContext().getPeriodLoader();
        return loader.loadPeriod(new XmlDataElement(perel), Term, PartOfTerm);
    }

    protected Notes readNotes(Element coursel) throws DbLoadException {
        NotesLoader loader = getContext().getNotesLoader();
        return loader.readNotes(new XmlDataElement(coursel));
    }
}
