/*
 *  Copyright (c) 2004, The University Scheduler Project
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of the University Scheduler Project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

package edu.rpi.scheduler.schedb.spec;

import edu.rpi.scheduler.schedb.DoubleRange;

import java.util.Collection;
import java.util.List;

/**
 * This is an actual Course object (which can be described by a 
 * {@code CourseDescriptor}).  Note that there is no department info here
 * as courses can span departments.  Department info is stored in the 
 * CourseDescriptor object, and multiple CourseDescriptors can point 
 * to the same course.  Also, each course has sections that actually have 
 * times and seats and such. 
 */
public interface Course extends Comparable<Course> {
    /**
     * Returns the name of the Course
     * @return String containing the name of the course
     */
    String getName();

    /**
     * Returns the course number for this course
     * @return The CourseID of this course
     */
    CourseID getNumber();
    
    /**
     * Returns a collection of all this course's {@code Section}s
     * @return A collection of all this course's {@code Section}s
     */
    Collection<Section> getSections();

	  /**
     * Returns a collection of all this course's {@code Section}s for a given Part Of Term
	 * @param  PoT the PartOfTerm Identifier
     * @return A collection of all this course's {@code Section}s
     */
    Collection<Section> getSectionsInPoT(String PoT);

	/**
     * Returns a {@code GradeType} that this course uses
     * @return A {@code GradeType} that this course uses
     */
    GradeType getGradeType();
    /**@return IntRange of possible credits one can earn in the course */
    DoubleRange getCredits();
    /**@return Notes for the current course*/
    Notes getNotes();
    /**
     * @param idstr String containing that section ID to be searched for
     * @return Section object for the section under this course that has the id passed
     */
    Section getSectionWithSectionIdString(String idstr);

	/**
	 * Returns a list of PartOfTerm Identifiers during which this course is offered
	 * @return List<String> of Part Of Term Identifiers
	 */
	List<String> getPartOfTermsOffered();
}
