package edu.rpi.scheduler.schedb.spec;

import edu.rpi.scheduler.DefensiveTools;
import edu.rpi.scheduler.schedb.Time;
import edu.rpi.scheduler.schedb.Duration;

import java.util.Arrays;

/**
 * This class is for representing the total week layout of each part of term.  It is a singleton.
 *
 * This class was created to fix the overloaded use of DailyTimePeriod.
 *
 * This class DOES NOT have information specific to one Term / PartOfTerm.  We asume all PartOfTerms have the same
 * start / end times for each day of the week. 
 */
public class TotalDailyTimePeriod {
    private static final String[] DAYNAMES = new String[] {
        "mon", "tue", "wed", "thu", "fri", "sat", "sun"
    };

    public static int getColumnFromDayNumber(TotalDailyTimePeriod tp, int day) {
        DefensiveTools.checkRange(day, "day", 0, 6);

        int realday = day;
        boolean[] days = tp.days;
        for (int i = 0; i < day; i++) {
            if (!days[i]) realday--;
        }
        return realday;
    }

    private final boolean[] days;
    private final TimePeriod period;

    private TotalDailyTimePeriod(int day, TimePeriod period) {
        this(day, period.getStart(), period.getEnd());
    }

    private TotalDailyTimePeriod(int day, TotalDailyTimePeriod period) {
        this(day, period.getPeriod());
    }

    private TotalDailyTimePeriod(int day, Time start, Time end) {
        if (day < 0 || day > 6) {
            throw new IllegalArgumentException("day must be 0-6 inclusive, but "
                    + "it was " + day);
        }

        boolean[] days = new boolean[7];
        days[day] = true;

        this.days = days;
        this.period = new TimePeriod(start, end);
    }

    private TotalDailyTimePeriod(boolean[] odays, Time start, Time end) {
        boolean[] days = odays.clone();
        if (days.length != 7) {
            throw new IllegalArgumentException("days must contain 7 elements "
                    + "(it contains " + days.length + ")");
        }

        this.days = days;
        this.period = new TimePeriod(start, end);
    }

	//Here we make this class a singleton so we always get the fully formed one back
	//TODO: Make this work
	private static TotalDailyTimePeriod INSTANCE;
	public static TotalDailyTimePeriod getInstance(boolean[] odays, Time start, Time end){
		if(INSTANCE == null){
			return INSTANCE = new TotalDailyTimePeriod(odays, start, end);
		}else{
			return INSTANCE;
		}
	}

	public boolean isOnDay(int day) { return days[day]; }

    public boolean[] getDays() { return days.clone(); }

    public int getDayCount() {
        int count = 0;
        for (boolean b : days) if (b) count++;
        return count;
    }

    /**
     * Returns the single day that this time period is on for, or {@code -1}
     * if this time period is not on any days or if it is on for more than one
     * day.
     */
    public int getDay() {
        int day = -1;
        int d = 0;
        for (boolean today : days) {
            if (today) {
                // if the day variable has already been set, then we must
                // return -1
                if (day != -1) return -1;
                day = d;
            }
            d++;
        }
        return day;
    }

    public TimePeriod getPeriod() { return period; }

    public Time getStart() { return period.getStart(); }

    public Time getEnd() { return period.getEnd(); }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof TotalDailyTimePeriod)) return false;

        final TotalDailyTimePeriod other = (TotalDailyTimePeriod) o;

        if (!Arrays.equals(getDays(), other.getDays())) return false;
        if (!getEnd().equals(other.getEnd())) return false;
        if (!getStart().equals(other.getStart())) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = getStart().hashCode();
        result = 29 * result + getEnd().hashCode();
        return result;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer(7*4);
        boolean first = true;
        boolean[] days = getDays();
        for (int i = 0; i < days.length; i++) {
            if (days[i]) {
                if (!first) sb.append(",");
                else first = false;

                sb.append(DAYNAMES[i]);
            }
        }

        return getStart() + "-" + getEnd() + " on " + sb;
    }

    public Duration getDuration() {
        return new Duration(getEnd().getMinutesFromMidnight()
                - getStart().getMinutesFromMidnight());
    }
}
