/*
 *  Copyright (c) 2004, The University Scheduler Project
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of the University Scheduler Project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

package edu.rpi.scheduler.ui.panels.courses;

import edu.rpi.scheduler.schedb.CourseDescriptor;
import edu.rpi.scheduler.schedb.spec.Year;
import edu.rpi.scheduler.engine.SelectedCourse;
import edu.rpi.scheduler.ui.SchedulingSession;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumnModel;
import javax.swing.*;
import java.util.*;
import java.util.List;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.*;

public class SelectedCoursesList extends AbstractTableModel {
    private static final List<String> COLUMN_NAMES = Arrays.asList("Extra", "Course", "Choose Terms");
    private static final List<Class<?>> COLUMN_CLASSES
            = Arrays.<Class<?>>asList(Boolean.class, String.class, JPanel.class);

    private List<SelectedCourseHolder> courses = new ArrayList<SelectedCourseHolder>();
    private List<SelectedCoursesListener> listeners = new ArrayList<SelectedCoursesListener>();
	private Map<CourseDescriptor, TermNarrowingPanel> cTNP = new HashMap<CourseDescriptor, TermNarrowingPanel>();

	private ConflictDetector conflictDetector =null;

	private final SchedulingSession session;
    public static final int COL_EXTRA = 0;
    public static final int COL_COURSE = 1;
	public static final int COL_TERMS = 2;
	
	public SelectedCoursesList(SchedulingSession session, ConflictDetector cd) {
        this.session = session;
		this.conflictDetector = cd;
	}

    public void addSelectedCoursesListener(SelectedCoursesListener l) {
        listeners.add(l);
    }

    public void removeSelectedCoursesListener(SelectedCoursesListener l) {
        listeners.remove(l);
    }

	/**
	 * This function updates all the icons in the SelectedCourseList.
	 */
	public void updateAllIcons(){
		for(SelectedCourseHolder holder:courses){
			holder.getTermNarrower().setButtonIcons(holder.getCourse(), this, conflictDetector);
		}
	}

	public boolean addCourse(CourseDescriptor cd, boolean extra) {
        for (SelectedCourseHolder holder : courses) {
            if (holder.getCourse().equals(cd)) return false;
        }

		//If we already created a panel for this course holder, use it
		TermNarrowingPanel tnp;
		if(cTNP.containsKey(cd)){
			tnp = cTNP.get(cd);
		}else{
			tnp = new TermNarrowingPanel(cd, session, conflictDetector);
		}
        SelectedCourseHolder holder = new SelectedCourseHolder(cd, extra, tnp, session);
		courses.add(holder);
		cTNP.put(cd, tnp);

		int ind = courses.size() - 1;
        fireTableRowsInserted(ind, ind);
        fireCourseAdded(holder);


		updateAllIcons();
		return true;
    }

    public void removeCourses(int[] rows) {
        List<SelectedCourse> courses = getCourses(rows);
        Collection<CourseDescriptor> cds = new ArrayList<CourseDescriptor>(courses.size());
        for (SelectedCourse course : courses) {
            cds.add(course.getCourse());
        }
		removeCourses(cds);
    }
    private void oldremoveCourses(int[] rows) {
        List<SelectedCourseHolder> removed = new ArrayList<SelectedCourseHolder>(rows.length);
        int off = 0;
        for (int i : rows) {
            int index = i - off;
            removed.add(courses.remove(index));
            fireTableRowsDeleted(index, index);
            off++;
        }
        fireCoursesRemoved(removed);
    }

    public void removeCourses(Collection<CourseDescriptor> toRemove) {
        Collection<SelectedCourseHolder> removed = new ArrayList<SelectedCourseHolder>();
        int i = 0;
        for (Iterator<SelectedCourseHolder> it = courses.iterator(); it.hasNext();) {
            SelectedCourseHolder holder = it.next();
            if (toRemove.contains(holder.getCourse())) {
                removed.add(holder);
                it.remove();
				cTNP.remove(holder.getCourse());

				//remove the course from our list of blocked sections too
				for(int k=0;k<Year.getInstance().getNumberOfTerms();k++){
					for(String PoT: Year.getInstance().getTerm(k).getPartOfTerms()){
						session.getEngine().UnblockCourseByTerm(holder.getCourse(), PoT);
					}
				}

				fireTableRowsDeleted(i, i);
                i--;
            }
            i++;
        }
		fireCoursesRemoved(removed);

		updateAllIcons();
	}


    public void removeCourse(CourseDescriptor cd) {
        int i = 0;
        for (Iterator<SelectedCourseHolder> it = courses.iterator();
             it.hasNext();) {
            SelectedCourseHolder holder = it.next();
            if (holder.getCourse().equals(cd)) {
                it.remove();
				cTNP.remove(holder.getCourse());
				
				//remove the course from our list of blocked sections too
				for(int k=0;k<Year.getInstance().getNumberOfTerms();k++){
					for(String PoT: Year.getInstance().getTerm(k).getPartOfTerms()){
						session.getEngine().UnblockCourseByTerm(holder.getCourse(), PoT);
					}
				}
				fireTableRowsDeleted(i, i);
                fireCourseRemoved(holder);
                break;
            }
            i++;
        }
		updateAllIcons();
	}
	public void setCourses(Collection<SelectedCourse> list) {
        int oldsize = courses.size();
        courses.clear();
        if (oldsize > 0) fireTableRowsDeleted(0, oldsize - 1);
        for (SelectedCourse course : list) {
			TermNarrowingPanel tnp;

			//If we already created a panel for this course holder, use it
			if(cTNP.containsKey(course.getCourse())){
				tnp = cTNP.get(course.getCourse());
			}else{
				tnp = new TermNarrowingPanel(course.getCourse(), session, conflictDetector);
			}
			SelectedCourseHolder holder = new SelectedCourseHolder(course.getCourse(), course.isExtra(), tnp, session);
			courses.add(holder);
			cTNP.put(course.getCourse(), tnp);
		}
        if (courses.size() > 0) fireTableRowsInserted(0, courses.size() - 1);

		fireCoursesChanged();

		updateAllIcons();
	}

    private void fireCourseAdded(SelectedCourseHolder holder) {
        fireCoursesAdded(Collections.singletonList(holder));
    }

    private void fireCoursesAdded(Collection<SelectedCourseHolder> list) {
        for (SelectedCoursesListener listener : listeners) {
            listener.coursesAdded(this, list);
        }
    }

    private void fireCourseRemoved(SelectedCourseHolder holder) {
        fireCoursesRemoved(Collections.singletonList(holder));
    }

    private void fireCoursesRemoved(Collection<SelectedCourseHolder> removed) {
        for (SelectedCoursesListener listener : listeners) {
            listener.coursesRemoved(this, removed);
        }
    }

    private void fireCoursesChanged() {
        for (SelectedCoursesListener listener : listeners) {
            listener.coursesChanged(this);
        }
    }

    private void fireExtraChanged(SelectedCourseHolder course,
            boolean newExtra) {
        for (SelectedCoursesListener listener : listeners) {
            listener.courseStatusChanged(this, course, newExtra);
        }
    }

    public Collection<SelectedCourse> getSelectedCourses() {
        Collection<SelectedCourse> list
                = new ArrayList<SelectedCourse>(courses.size());
        for (SelectedCourseHolder courseHolder : courses) {
            list.add(new SelectedCourse(courseHolder.getCourse(),
                    courseHolder.isExtra()));
        }
        return list;
    }

    public List<SelectedCourse> getCourses(int[] rows) {
        List<SelectedCourse> list = new ArrayList<SelectedCourse>(rows.length);
        for (int i : rows) {
            SelectedCourseHolder holder = courses.get(i);
            list.add(new SelectedCourse(holder.getCourse(), holder.isExtra()));
        }
        return list;
    }

    public CourseDescriptor getCourse(int row) {
        return courses.get(row).getCourse();
    }

    public boolean containsCourse(CourseDescriptor cd) {
        for (SelectedCourseHolder holder : courses) {
            if (holder.getCourse().equals(cd)) return true;
        }
        return false;
    }

    public String getColumnName(int column) {
        return COLUMN_NAMES.get(column);
    }

    public int findColumn(String columnName) {
        return COLUMN_NAMES.indexOf(columnName);
    }

    public Class<?> getColumnClass(int columnIndex) {
        return COLUMN_CLASSES.get(columnIndex);
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return (columnIndex == COL_EXTRA);
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        if (columnIndex != COL_EXTRA) {
            throw new IllegalArgumentException("can't modify column "
                    + columnIndex);
        }
        boolean newExtra = (Boolean) aValue;
        SelectedCourseHolder course = courses.get(rowIndex);
        if (course.isExtra() == newExtra) return;

        course.setExtra(newExtra);
        fireTableCellUpdated(rowIndex, columnIndex);
        fireExtraChanged(course, newExtra);
    }

    public int getRowCount() { return courses.size(); }

    public int getColumnCount() { return 3; }

    public Object getValueAt(int rowIndex, int columnIndex) {
        SelectedCourseHolder cd = courses.get(rowIndex);
        if (columnIndex == COL_COURSE) {
            return cd.getCourse().getActualCourse().getName();
        } else if (columnIndex == COL_EXTRA) {
            return cd.isExtra();
        } else if (columnIndex == COL_TERMS) {
            return cd.getTermNarrower();
        } else {
            throw new IllegalArgumentException("no such column " + columnIndex);
        }
    }

    public int getIndexOf(CourseDescriptor course) {
        int index = 0;
        for (SelectedCourseHolder holder : courses) {
            if (holder.getCourse().equals(course)) return index;
            index++;
        }
        return -1;
    }

	/**
	 * This sets the conflictDetector that this SelectedCourseList will use to determine
	 * what Part Of Terms to set as available for selection
	 * @param conflictDetector the conflictDetector we want to use.  This should be the same one
	 * as the PossibleCoursesListModel uses for behavior that is what is expected
	 */
	public void setConflictDetector(ConflictDetector conflictDetector) {
		this.conflictDetector = conflictDetector;
	}

	/**
	 * This class holds the info contained in a single row of the SelectedCourseList.
	 * This includes a TermNarrowingPanel.
	 */
	public static class SelectedCourseHolder {
        private final CourseDescriptor course;
        private boolean extra;
		private TermNarrowingPanel TermNarrower;
		private SchedulingSession session;

		public SelectedCourseHolder(CourseDescriptor course, boolean extra, TermNarrowingPanel tnp, SchedulingSession session) {
            this.course = course;
            this.extra = extra;
			this.session = session;
			this.TermNarrower = tnp;
            this.TermNarrower.setBackground(Color.WHITE);
        }

        public CourseDescriptor getCourse() { return course; }

        public boolean isExtra() { return extra; }

        private void setExtra(boolean extra) { this.extra = extra; }

		public TermNarrowingPanel getTermNarrower(){return TermNarrower;}

		public String toString() {
            return "SelectedCourseHolder: " +
                    "course=" + course +
                    ", extra=" + extra;
        }

	}

	/**
	 * MouseListener to forward actions to the buttons.
	 *
	 * Code extended from code taken from:
	 * http://www.devx.com/getHelpOn/10MinuteSolution/20425/1763
	 */
	public static class JTableButtonMouseListener implements MouseListener {
		private JTable __table;

		private void __forwardEventToButton(MouseEvent e) {
			if(SwingUtilities.isLeftMouseButton(e) && e.getClickCount()==1){
				TableColumnModel columnModel = __table.getColumnModel();
				int column = columnModel.getColumnIndexAtX(e.getX());
				int row    = e.getY() / __table.getRowHeight();
				Object value;
				MouseEvent buttonEvent;

				if(row >= __table.getRowCount() || row < 0 ||
				   column >= __table.getColumnCount() || column < 0)
					return;

				Rectangle panelBounds = __table.getCellRect(row, column, false);
				value = __table.getValueAt(row, column);
				if(!(value instanceof TermNarrowingPanel))
				  return;
				List<List<JButton>> buttons = ((TermNarrowingPanel) value).getButtons();
				int i=0;
				for(List<JButton> bl: buttons){
					int j=0;
					for(JButton b:bl){
						Rectangle buttonBounds = b.getBounds();
						buttonBounds.translate(panelBounds.x, panelBounds.y);
						if (buttonBounds.contains(e.getX(), e.getY())){  // The click was on this button.
							buttonEvent =
							  (MouseEvent)SwingUtilities.convertMouseEvent(__table, e, b);
							//b.dispatchEvent(buttonEvent);
							//b.dispatchEvent(new MouseEvent(b, MouseEvent.MOUSE_RELEASED, e.getWhen(), e.getModifiers(), 0, 0, e.getClickCount(), e.isPopupTrigger(), e.getButton()));
							b.doClick();

							((SelectedCoursesList) __table.getModel()).updateAllIcons();
							// This is necessary so that when a button is pressed and released
							// it gets rendered properly.  Otherwise, the button may still appear
							// pressed down when it has been released.
							__table.repaint();
						}
						j++;
					}
					i++;
				}
			}
		}

		public JTableButtonMouseListener(JTable table) {
			__table = table;
		}

		public void mouseClicked(MouseEvent e) {
			__forwardEventToButton(e);
		}

		public void mouseEntered(MouseEvent e) {
			__forwardEventToButton(e);
		}

		public void mouseExited(MouseEvent e) {
			__forwardEventToButton(e);
		}

		public void mousePressed(MouseEvent e) {
			__forwardEventToButton(e);
		}

		public void mouseReleased(MouseEvent e) {
			__forwardEventToButton(e);
		}
	}

}
