/*
 *  Copyright (c) 2004, The University Scheduler Project
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of the University Scheduler Project nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 */

package edu.rpi.scheduler.ui.widgets;

import edu.rpi.scheduler.schedb.spec.Schedule;
import edu.rpi.scheduler.schedb.UniqueSection;
import edu.rpi.scheduler.ui.SchedulingSession;
import edu.rpi.scheduler.DefensiveTools;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.util.Collection;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;

/**
 * This class represents a single week of SchduleGrid.  This grid is used
 * in the SchduleViewer to show a week's worth of class.
 */
public abstract class ScheduleGrid extends TimeGrid {
    public static final String PROP_SELECTED_SCHEDULE = "selectedSchedule";

    private PropertyChangeSupport pcs = new PropertyChangeSupport(this);

    private TimeGridSchedulePainter schedPainter;
    private Schedule schedule = null;

    protected ScheduleGrid() {
        this(null, null);
    }

    protected ScheduleGrid(SchedulingSession session) {
        this(session, null);
    }

    protected ScheduleGrid(SchedulingSession session,
            TimeGridSchedulePainter schedPainter) {
        super(session);
        if (schedPainter == null) {
            this.schedPainter = new TimeGridSchedulePainter(session, this);
        } else {
            this.schedPainter = schedPainter;
        }
    }

	/**
	 * This sets the sections that should be in this Schedule Grid
	 * @param sections A collection of UniqueSection that this grid should have
	 */
	protected synchronized void setSections(Collection<UniqueSection> sections) {
        DefensiveTools.checkNull(sections, "sections");

        this.schedule = null;
        schedPainter.setSession(getSession());
        schedPainter.setSections(sections);

        repaint();
    }

	/**
	 * This function takes a Schedule that we want to display.  It will
	 * display everything in this schedule regardless of if it's in the
	 * term / part of term that this ScheduleGrid represents.  The way to
	 * make things work properly is to only call this function with a
	 * schedule that contains only the sections that you want to show in
	 * this Term / Part Of Term.  This has to be this way because
	 * the ScheduleGrid itself does not know or care what Term/Part Of Term
	 * it is.
	 * @param schedule Schedule with data for only the term / Part Of Term you want
	 * to show.
	 */
	protected synchronized void setSchedule(Schedule schedule) {

		if (this.schedule == schedule) return;
        
        this.schedule = schedule;
        schedPainter.setSession(getSession());
        schedPainter.setSchedule(schedule);

	}

	/**
	 * This function sets the border around all the blocks for a section when
	 * it is selected.  Null is passed it removes the border from all blocks.
	 *
	 * This shouts up to the ScheduleViewer (which is listening for this) which
	 * then calls down to all it's children to update accordingly
	 *
	 * @param section the UniqueSection that we want to highlight
	 */
	public void setSelectedSection(UniqueSection section) {
        boolean repaint = false;
        for (VisualUnique vu : schedPainter.getSections()) {
            boolean oldSelected = vu.isSelected();
            boolean newSelected = vu.getUniqueSection().equals(section);
            if (oldSelected != newSelected) {
                repaint = true;
                vu.setSelected(newSelected);
				if(section != null){
					pcs.firePropertyChange(PROP_SELECTED_SCHEDULE, null, section);
				}
			}
        }
        if (repaint) repaint();

    }

    public void addSelectedSectionListener(PropertyChangeListener l) {
        pcs.addPropertyChangeListener(PROP_SELECTED_SCHEDULE, l);
    }

    public void removeSelectedSectionListener(PropertyChangeListener l) {
        pcs.removePropertyChangeListener(PROP_SELECTED_SCHEDULE, l);
    }

    /**
     * May return null if no schedule is set, just sections
     */
    public Schedule getSchedule() { return schedule; }

    public final TimeGridSchedulePainter getSchedulePainter() {
        return schedPainter;
    }

    protected final void setSchedulePainter(TimeGridSchedulePainter schedPainter) {
        this.schedPainter = schedPainter;
    }

    public synchronized void paintComponent(Graphics og) {
        super.paintComponent(og);

        Graphics2D g = (Graphics2D) og;

        updateGridDimensions();

        paintGridBG(g);
        paintBetweenGrid(g);
        paintGridFG(g);

        // draw boxes
        schedPainter.paint(g);
    }

    protected synchronized void updateGridDimensions() {
        int gridWidth = getWidth();
        int gridHeight = getHeight();

        initGrid(gridWidth, gridHeight);
    }

    protected void paintBetweenGrid(Graphics2D g) {
        // does nothing
    }
}
