package edu.rpi.scheduler.ui.widgets;

import edu.rpi.scheduler.ui.SchedulingSession;
import edu.rpi.scheduler.ui.TimeGridType;
import edu.rpi.scheduler.schedb.spec.Schedule;
import edu.rpi.scheduler.schedb.spec.Year;
import edu.rpi.scheduler.schedb.*;
import edu.rpi.scheduler.engine.SchedulerEngine;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;

import java.util.*;
import java.util.List;
import java.beans.PropertyChangeListener;
import java.text.NumberFormat;


/**
 * This class represents collection of Detailed Schedule Grids and is a Singleton!!!
 *
 * This represents an entire year.  This object has a DetailedScheduleGrid for each term/part of term. Use YearDetailedScheduleGrid.getInsance() to get a useable grid.  (this is done to ensure
 * it is properly populated with the right number of terms and part of terms)
 *
 * This class mostly just pushes function calls down to its children or vice versa
 *
 */
public class YearDetailedScheduleGrid extends JPanel{

    private final NumberFormat format = NumberFormat.getNumberInstance();
    {
        format.setMinimumIntegerDigits(2);
    }

    List<List<DetailedScheduleGrid>> myGrids = new LinkedList<List<DetailedScheduleGrid>>();
    private List<List<JLabel>> myGridsLabel = new LinkedList<List<JLabel>>();
	private Map<UniqueSection,Paint> paints;
    static YearDetailedScheduleGrid INSTANCE;
	SchedulingSession session;

    public static YearDetailedScheduleGrid getInstance(){
		if(INSTANCE == null){
			INSTANCE = new YearDetailedScheduleGrid();
		}
		return INSTANCE;
	}

	public static YearDetailedScheduleGrid getInstance(SchedulingSession session){
		if(INSTANCE == null){
			INSTANCE = new YearDetailedScheduleGrid(session);
		}
		return INSTANCE;
	}

    private YearDetailedScheduleGrid() {
        this(null);
    }

	private YearDetailedScheduleGrid(SchedulingSession session){
		this.session = session;
		populatemygrids(session);
        populatemyGridsLabel();
	}



    protected TimeGridType getTimeGridType() {
         return TimeGridType.SCHEDULE;
    }

	/**
	 * This function takes and populates our two dimentional list with
	 * the proper number of DetailedScheduleGrids for our current setup.
	 * @param session SechedulingSession that we are using for this object
	 * (so we have the right number of terms and part of terms) 
	 */
	private void populatemygrids(SchedulingSession session){
		int i=0;

		while(i < Year.getInstance().getNumberOfTerms()){
			myGrids.add(new LinkedList<DetailedScheduleGrid>());
			int j=0;
			while(j < Year.getInstance().getTerm(i).getNumberOfPartOfTerms()){
				myGrids.get(i).add(new DetailedScheduleGrid(session, paints));
				j++;
			}
			i++;
        }      
    }


	/**
	 * This populates the GridsLabel that shows what Term / Part Of Term you are showing
	 */
	private void populatemyGridsLabel(){
        for (int i = 0; i < Year.getInstance().getNumberOfTerms(); i++) {
            List<JLabel> internalJLabelList = new LinkedList();
            for (int j = 0; j < Year.getInstance().getTerm(i).getNumberOfPartOfTerms(); j++) {
                internalJLabelList.add(new JLabel(Year.getInstance().getTerm(i).getPartOfTerm(j).getIdentifier()));
            }
            myGridsLabel.add(internalJLabelList);
        }
    }


	/**
	 * This sets the session that this panel is part of.
	 *
	 * Essentially this session is pushed down to all our sub panels.
	 * 
	 * @param session SchedulingSession this panel is being used for
	 */
	public void setSession(SchedulingSession session){
		this.session = session;
		for(List<DetailedScheduleGrid> lt: myGrids){
			for(DetailedScheduleGrid l: lt){
				l.setSession(session);
			}
		}

	}

	/**
	 * This gives this panel a Schedule to display.
	 *
	 * In this function we split this complete year Schedule
	 * into a bunch of schedules that only actually have sections
	 * that meet in a single part of term.  These split up schedules
	 * are then passed to each of our DetailedScheduleGrids.
	 * This is not the most elegant way of doing this, but it is
	 * effective and simple.
	 * @param s Schedule that we want to display.
	 */
	public void setSchedule(Schedule s){

		int i=0;

		for(List<DetailedScheduleGrid> lt: myGrids){
			int j=0;
			for(DetailedScheduleGrid l: lt){
				//generate list of Unique sections in this part of Term
				List<UniqueSection> sectionsThisPoT = new LinkedList();
				sectionsThisPoT.addAll(s.getSections(Year.getInstance().getTerm(i).getIdentifier(), Year.getInstance().getTerm(i).getPartOfTerm(j).getIdentifier()));
				//push this list into a new schedule
				//Schedule localSchedule = new DefaultSchedule(s.getSchedulerPlugin());
				//for(UniqueSection us:sectionsThisPoT){
				//	localSchedule.addSection(us);
				//}

				myGrids.get(i).get(j).setSections(sectionsThisPoT);
				//pass this schedule to the correct grid
				//myGrids.get(i).get(j).setSchedule(localSchedule);
				j++;
			}
			i++;
		}
	}

	/**
	 * This function sets what section is selected.
	 *
	 * Basicly it calls down to each of the DetailedScheduleGrid and passes this info
	 * down.
	 * @param section UniqueSection that is selected
	 */
	public void setSelectedSection(UniqueSection section){
		List<String> Term;
		//figure out what terms/part of terms this section meets in
		if(section != null){
			Term = section.getTimeMask().getActiveTerms();
		}else{
			return;
		}

		//Clean out each of the terms
		int i=0;
		for(List<DetailedScheduleGrid> lt: myGrids){
			int j=0;
			for(DetailedScheduleGrid l: lt){
				myGrids.get(i).get(j).setSelectedSection(null);
				j++;
			}
			i++;
		}
		//push the correct section down to each DetailedScheduleGrid
		for(String t:Term){
			List<String> PartOfTerm = section.getTimeMask().getTerm(t).getActivePartOfTerms();
			for(String PoT:PartOfTerm){
				//pass this setSelectedSection down to each of the appropriate terms/part of terms that need it
				myGrids.get(Year.getInstance().getTerm(t).getNumber()).get(Year.getInstance().getTerm(t).getPoTNumber(PoT)).setSelectedSection(section);
			}
		}
	}

	public TimeGridSchedulePainter getSchedulePainter(int term, int PoT){
		return myGrids.get(term).get(PoT).getSchedulePainter();
	}

	/**
	 * Returns the paint that we are using for this section.
	 * @param section UniqueSection we want paint for
	 * @return Paint that is being used to color it
	 */
	public Paint getPaint(UniqueSection section) {
		return paints.get(section);
	}

	 public void addSelectedSectionListener(PropertyChangeListener l) {
		int i=0;

		for(List<DetailedScheduleGrid> lt: myGrids){
			int j=0;
			for(DetailedScheduleGrid lsg: lt){
				myGrids.get(i).get(j).addSelectedSectionListener(l);
				j++;
			}
			i++;
		}
	}

	/**
	 * This function sets the times that we do not want to have class.
	 *
	 * This function takes a YearMask of blocked time and splits it up
	 * into PartOfTerm masks (WeekMasks) that each cover one of our
	 * DetailedScheduleGrids
	 *
	 * @param blockedTime YearMask of times we do not want to have class
	 */
	public void setBlockedTime(YearMask<?> blockedTime) {
		int i=0;

		for(List<DetailedScheduleGrid> lt: myGrids){
			int j=0;
			for(DetailedScheduleGrid lsg: lt){
				myGrids.get(i).get(j).setBlockedTime(blockedTime.getTerm(i).getPartOfTerm(j));
				j++;
			}
			i++;
		}
	}
	/**
	 * This function generates the grid of DetailedScheduleGrids
	 */
    public JComponent CombinedDisplayYearTimeSelectorOverlay()
    {
        setLayout(new BorderLayout());
        JPanel p = new JPanel();
        OverlayLayout overlay = new OverlayLayout(p);
        p.setLayout(overlay);
        p.add(DisplayYearGrid(), 0);
        p.add(DisplayYearSGOverylay(), 1);


        add(p, BorderLayout.CENTER);

        return p;
    }
    public JComponent DisplayYearGrid(){
        JPanel YDSG = new JPanel();
        YDSG.setLayout(new GridLayout(0,Year.getInstance().getMaxNumberOfPartOfTerms()));

		List<List<DetailedScheduleGrid>> dsgList = new LinkedList<List<DetailedScheduleGrid>>();

		int i=0;

		for(java.util.List<DetailedScheduleGrid> lt: myGrids){
			int j=0;
			List<DetailedScheduleGrid> localList = new LinkedList<DetailedScheduleGrid>();
			for(DetailedScheduleGrid lsg: lt){
				localList.add(myGrids.get(i).get(j));
				j++;
			}
			dsgList.add(localList);
			i++;
		}

		i=0;
		for(List<DetailedScheduleGrid> lt:dsgList){
			int j=0;

			for(DetailedScheduleGrid lsg: lt){
				dsgList.get(i).get(j).setVisible(true);
                dsgList.get(i).get(j).setOpaque(false);
                YDSG.add(dsgList.get(i).get(j));
				j++;
			}
			i++;
		}

		YDSG.setVisible(true);
        YDSG.setOpaque(false);
        return YDSG;
	}

    public static int PLAIN;
    public static int CENTER;
    public JPanel DisplayYearSGOverylay(){
        JPanel YSGO = new JPanel();
        YSGO.setLayout(new GridLayout(0,Year.getInstance().getMaxNumberOfPartOfTerms()));

        //YSGO.setSize(YSGO.getMaximumSize());
        YSGO.setVisible(false);
        YSGO.setOpaque(true);
        int i=0;
        while(i < Year.getInstance().getNumberOfTerms()){
            int j=0;
            while(j < Year.getInstance().getTerm(i).getNumberOfPartOfTerms()){
                myGridsLabel.get(i).get(j).setVisible(true);
                myGridsLabel.get(i).get(j).setFont(new Font("Arial",PLAIN,40));
                myGridsLabel.get(i).get(j).setHorizontalAlignment(CENTER);
                myGridsLabel.get(i).get(j).setOpaque(false);
                YSGO.add(myGridsLabel.get(i).get(j));
                j++;
            }
            i++;
        }
        YSGO.setVisible(true);
        YSGO.setBackground(Color.WHITE);
        return YSGO;
    }


	/**
	 * This calls down to each of our DetailedScheduleGrids and updates the
	 * schedule list within each of them.  We also need to make sure we setup
	 * our paints here so we have a color for each course, and make sure we
	 * pass the new paints to each of our children.
	 */
	public void updateScheduleList() {
		setupPaints(session);

		int i=0;

		for(List<DetailedScheduleGrid> lt: myGrids){
			int j=0;
			for(DetailedScheduleGrid lsg: lt){
				myGrids.get(i).get(j).setPaints(paints);
				myGrids.get(i).get(j).updateScheduleList();
				j++;
			}
			i++;
		}
	}

	public List<TimeGridSchedulePainter> getAllSchedulePainter() {
		int i=0;
		List<TimeGridSchedulePainter> SchedulePainters = new LinkedList<TimeGridSchedulePainter>();

		for(java.util.List<DetailedScheduleGrid> lt: myGrids){
			int j=0;
            for(DetailedScheduleGrid lsg: lt){
				SchedulePainters.add(myGrids.get(i).get(j).getSchedulePainter());
				j++;
			}
			i++;
		}
		return SchedulePainters;
	}

	public WeekMask<?> getBlockedTime(SchedulerEngine e, DetailedScheduleGrid dsg) {
		YearMask<?> ymask = null;
		ymask = e.getBlockedTime();
		int i=0;

		for(List<DetailedScheduleGrid> lt: myGrids){
			int j=0;
			for(DetailedScheduleGrid lsg: lt){
				if(myGrids.get(i).get(j).equals(dsg)){
					//if we're here we have the detailed schedule grid that called us
					return ymask.getTerm(i).getPartOfTerm(j);
				}
				j++;
			}
			i++;
		}
		//we should never get here
		return null;
	}


	// This stuff is for making colors global across terms
	
	static final Color[] COLORS_INITIAL = new Color[] {
        new Color(226, 174, 236, 200), new Color(249, 217, 184, 200),
        new Color(249, 173, 173, 200), new Color(167, 241, 135, 200),
        new Color(237, 243, 255, 200), new Color(162, 202, 243, 200),
        new Color(255, 247, 153, 200),
    };
    private static final Random random = new Random();

	/**
	 * This function sets up a map of Courses to colors so that each course has the same color regardless
	 * of what term it is being displayed in.
	 * @param session SchedulingSession we are working in. 
	 */
	private void setupPaints(SchedulingSession session){

		LinkedList<Color> colors = new LinkedList<Color>(
				Arrays.asList(COLORS_INITIAL));
		Collections.shuffle(colors, random);

		SchedulerEngine scheduler = session.getEngine();
		List<Schedule> schedules = scheduler.getGeneratedSchedules();

		if (schedules.size() > 0) {
			Collection<CourseDescriptor> courses = scheduler.getSelectedCourseDescriptors();
			Map<UniqueSection,Paint> paints;
			paints = new HashMap<UniqueSection, Paint>(courses.size());
			for (CourseDescriptor course : courses) {
				Color color;
				if (colors.size() == 0) {
					//TOMAYBE: colors should never be too close to other colors
					color = getRandomLightColor();
				} else {
					color = colors.removeFirst();
				}
				Paint paint;
				if (scheduler.isExtra(course)) paint = new PolkaPaint(color);
				else paint = color;

				for (Schedule schedule : schedules) {
					for (UniqueSection section : schedule.getSections()) {
						if (section.getCourseDescriptor().equals(course)) {
							paints.put(section, paint);
						}
					}
				}
			}
			this.paints = paints;
		}
	}

	static Color getRandomLightColor() {
        return new Color(128+random.nextInt(128),
                128+random.nextInt(128),
                128+random.nextInt(128));
    }
	
	protected static class PolkaPaint extends TexturePaint {
        private static final int DOT_SIZE = 4;
        private static final int TEXTURE_SIZE = DOT_SIZE*2;
        private static final Rectangle RECT = new Rectangle(TEXTURE_SIZE, TEXTURE_SIZE);

        private final Color color;

        public PolkaPaint(Color color) {
            super(genSlashyImg(color), RECT);

            this.color = color;
        }

        public Color getColor() { return color; }

        private static BufferedImage genSlashyImg(Color color) {
            BufferedImage img = new BufferedImage(
                    TEXTURE_SIZE, TEXTURE_SIZE, BufferedImage.TYPE_3BYTE_BGR);


            Graphics2D g = img.createGraphics();
            g.setBackground(color);
            g.clearRect(0, 0, img.getWidth(), img.getHeight());

            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);

            g.setColor(Color.WHITE);
            g.drawLine(0, 0, img.getWidth(), img.getHeight());

            return img;
        }

        private static BufferedImage genStippledImg(Color color) {
            BufferedImage img = new BufferedImage(
                    2, 2, BufferedImage.TYPE_3BYTE_BGR);

            img.setRGB(0, 0, 0xffffffff);
            img.setRGB(1, 1, 0xffffffff);
            int crgb = color.getRGB();
            img.setRGB(0, 1, crgb);
            img.setRGB(1, 0, crgb);

            return img;
        }

        private static BufferedImage genPolkaImg(Color color) {
            BufferedImage img = new BufferedImage(
                    TEXTURE_SIZE, TEXTURE_SIZE, BufferedImage.TYPE_4BYTE_ABGR);

            Graphics2D g = img.createGraphics();

            g.setBackground(color);
            g.clearRect(0, 0, img.getWidth(), img.getHeight());

            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);

            g.setColor(Color.WHITE);
            g.fillOval(0, 0, DOT_SIZE, DOT_SIZE);
            g.fillOval(DOT_SIZE, DOT_SIZE, DOT_SIZE, DOT_SIZE);
            return img;
        }
    }
}
